/* /////////////////////////////////////////////////////////////////////////////
 * File:        atlstl_string_access.h
 *
 * Purpose:     Contains classes and functions for dealing with OLE/COM strings.
 *
 * Created:
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/atlstl
 *                          http://www.atlstl.org/
 *
 *              email:      submissions@atlstl.org  for submissions
 *                          admin@atlstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _ATLSTL_INCL_H_ATLSTL_STRING_ACCESS
#define _ATLSTL_INCL_H_ATLSTL_STRING_ACCESS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _ATLSTL_VER_H_ATLSTL_STRING_ACCESS_MAJOR     1
#define _ATLSTL_VER_H_ATLSTL_STRING_ACCESS_MINOR     3
#define _ATLSTL_VER_H_ATLSTL_STRING_ACCESS_REVISION  2
#define _ATLSTL_VER_H_ATLSTL_STRING_ACCESS_EDIT      33
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _ATLSTL_INCL_H_ATLSTL
 #include "atlstl.h"    // Include the ATLSTL root header
#endif /* !_ATLSTL_INCL_H_ATLSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The ATLSTL components are contained within the atlstl namespace. This is
 * actually an alias for stlsoft::atlstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _ATLSTL_NO_NAMESPACE    atlstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::atlstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             atlstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _ATLSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::atlstl */
namespace atlstl
{
 #else
/* Define stlsoft::atlstl_project */

namespace stlsoft
{

namespace atlstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_string_access String Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types

/// \defgroup atlstl_string_access_shims String Access Shims (ATLSTL)
/// \ingroup ATLSTL concepts_shims_string_access
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/* CWindow */

#ifdef __ATLWIN_H__
/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the window text of a given window
/// may be accessed as a null-terminated string.
class c_str_ptr_null_CWindow_proxy
{
private:
    typedef c_str_ptr_null_CWindow_proxy    class_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given CWindow instance
    ///
    /// \param w The CWindow instance from which the text will be retrieved
    as_explicit_k c_str_ptr_null_CWindow_proxy(CWindow const &w)
    {
        int length  =   w.GetWindowTextLength();

        if(length == 0)
        {
            m_buffer = 0;
        }
        else
        {
            m_buffer = new TCHAR[1 + length];

            w.GetWindowText(m_buffer, length + 1);
        }
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_null_CWindow_proxy(class_type &rhs)
        : m_buffer(rhs.m_buffer)
    {
        rhs.m_buffer = 0;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_null_CWindow_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the window contents, or
    /// NULL if the window contains no text.
    operator LPCTSTR () const
    {
        return m_buffer;
    }

// Members
private:
    LPTSTR  m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr() function, such that the window text of a given window may be
/// accessed as a null-terminated string.
class c_str_ptr_CWindow_proxy
{
private:
    typedef c_str_ptr_CWindow_proxy class_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given CWindow instance
    ///
    /// \param w The CWindow instance from which the text will be retrieved
    as_explicit_k c_str_ptr_CWindow_proxy(CWindow const &w)
    {
        int length  =   w.GetWindowTextLength();

        m_buffer = new TCHAR[1 + length];

        w.GetWindowText(m_buffer, length + 1);
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_CWindow_proxy(class_type &rhs)
        : m_buffer(rhs.m_buffer)
    {
        rhs.m_buffer = 0;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_CWindow_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the window contents, or
    /// the empty string "" if the window contains no text.
    operator LPCTSTR () const
    {
        return m_buffer;
    }

// Members
protected:
    LPTSTR  m_buffer;

// Not to be implemented
protected:
    void operator =(class_type const &rhs);
};
#endif /* __ATLWIN_H__ */

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr_null
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or NULL.
 */

/* CWindow */
#ifdef __ATLWIN_H__
/// \brief Returns the corresponding C-string pointer of the CWindow \c w, or a null pointer
inline c_str_ptr_null_CWindow_proxy c_str_ptr_null(atlstl_ns_qual_atl(CWindow) const &w)
{
    return c_str_ptr_null_CWindow_proxy(w);
}
#endif /* __ATLWIN_H__ */

/* CComBSTR */
/// \brief Returns the corresponding C-string pointer of the CComBSTR \c s, or a null pointer
inline LPCOLESTR c_str_ptr_null(atlstl_ns_qual_atl(CComBSTR) const &s)
{
    /* NULL is a valid BSTR value, so may return that */
    return s.m_str;
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or to an empty string.
 */

/* CWindow */
#ifdef __ATLWIN_H__
/// \brief Returns the corresponding C-string pointer of the CWindow \c w
inline c_str_ptr_CWindow_proxy c_str_ptr(atlstl_ns_qual_atl(CWindow) const &w)
{
    return c_str_ptr_CWindow_proxy(w);
}
#endif /* __ATLWIN_H__ */

/* CComBSTR */
/// \brief Returns the corresponding C-string pointer of the CComBSTR \c s
inline LPCOLESTR c_str_ptr(atlstl_ns_qual_atl(CComBSTR) const &s)
{
    /* NULL is a valid BSTR value, so check for that */
    return (s.m_str != 0) ? s.m_str : L"";
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_len
 *
 * This can be applied to an expression, and the return value is the number of
 * characters in the character string in the expression.
 */

/* CWindow */
#ifdef __ATLWIN_H__
/// \brief Returns the length (in characters) of the contents of the window \c w, <b><i>not</i></b> including the null-terminating character
inline as_size_t c_str_len(atlstl_ns_qual_atl(CWindow) const &w)
{
    return static_cast<as_size_t>(w.GetWindowTextLength());
}
#endif /* __ATLWIN_H__ */

/* CComBSTR */
/// \brief Returns the length (in characters) of the CComBSTR \c s, <b><i>not</i></b> including the null-terminating character
inline as_size_t c_str_len(atlstl_ns_qual_atl(CComBSTR) const &s)
{
    return s.Length();
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_size
 *
 * This can be applied to an expression, and the return value is the number of
 * bytes required to store the character string in the expression, NOT including
 * the null-terminating character.
 */

/* CWindow */
#ifdef __ATLWIN_H__
/// \brief Returns the size (in bytes) of the contents of the window \c w, <b><i>not</i></b> including the null-terminating character
inline as_size_t c_str_size(atlstl_ns_qual_atl(CWindow) const &w)
{
    return c_str_len(w) * sizeof(TCHAR);
}
#endif /* __ATLWIN_H__ */

/* CComBSTR */
/// \brief Returns the size (in bytes) of the CComBSTR \c s, <b><i>not</i></b> including the null-terminating character
inline as_size_t c_str_size(atlstl_ns_qual_atl(CComBSTR) const &s)
{
    return c_str_len(s) * sizeof(OLECHAR);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group atlstl_string_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _ATLSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace atlstl
 #else
} // namespace atlstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_ATLSTL_INCL_H_ATLSTL_STRING_ACCESS */

/* ////////////////////////////////////////////////////////////////////////// */
