/* /////////////////////////////////////////////////////////////////////////////
 * File:        comstl_enum_sequence.h (originally MOEnSeq.h, ::SynesisCom)
 *
 * Purpose:     STL sequence for IEnumXXXX enumerator interfaces.
 *
 * Created:     17th September 1998
 * Updated:     22nd August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_ENUM_SEQUENCE
#define _COMSTL_INCL_H_COMSTL_ENUM_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_ENUM_SEQUENCE_MAJOR    2
#define _COMSTL_VER_H_COMSTL_ENUM_SEQUENCE_MINOR    3
#define _COMSTL_VER_H_COMSTL_ENUM_SEQUENCE_REVISION 1
#define _COMSTL_VER_H_COMSTL_ENUM_SEQUENCE_EDIT     141
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                    // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */
#ifndef _COMSTL_INCL_H_COMSTL_REFCOUNT_FUNCTIONS
 #include "comstl_refcount_functions.h" // safe_release, release_set_null
#endif /* !_COMSTL_INCL_H_COMSTL_REFCOUNT_FUNCTIONS */
#include <algorithm>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup comstl_itf_library Interface Library
/// \ingroup COMSTL libraries
/// \brief This library provides facilities for manipulating COM interfaces
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Policy type that causes COM enumerator cloning according the STL Input Iterator concept
///
/// \param I The enumeration interface
template<cs_typename_param_k I>
struct input_cloning_policy
{
public:
    typedef I   interface_type;

public:
    /// "Clones" the given COM enumerator interface according to the Input Iterator concept
    static interface_type *clone(interface_type *&src)
    {
        interface_type *ret =   src;

        src = 0;

        return ret;
    }
};

/// Policy type that causes COM enumerator cloning according the STL Forward Iterator concept
///
/// \param I The enumeration interface
template<cs_typename_param_k I>
struct forward_cloning_policy
{
public:
    typedef I   interface_type;

public:
    /// "Clones" the given COM enumerator interface according to the Forward Iterator concept
    static interface_type *clone(interface_type const *src)
    {
        interface_type  *ret;

        if( src == 0 ||
            FAILED(const_cast<interface_type*>(src)->Clone(&ret)))
        {
            ret = 0;
        }

        return ret;
    }
};

/// Policy type that causes COM enumerator cloning according the best available STL Iterator concept
///
/// \param I The enumeration interface
template<cs_typename_param_k I>
struct degenerate_cloning_policy
{
public:
    typedef I   interface_type;

public:
    /// "Clones" the given COM enumerator interface according to the best available concept
    static interface_type *clone(interface_type *&src)
    {
        interface_type  *ret;

        if(src == 0)
        {
            ret = 0;
        }
        else if(FAILED(src->Clone(&ret)))
        {
            ret = src;
            src = 0;
        }

        return ret;
    }
};

/// Adapts a value policy to a function object based interface
///
/// \param P The value policy (e.g. BSTR_policy, VARIANT_policy, FORMATETC_policy)
template <cs_typename_param_k P>
struct policy_adaptor
    : public P
{
public:
    typedef cs_typename_type_k P::value_type    value_type;

public:
    /// The initialisation function object type
    struct _init
    {
        /// The function call operator, which causes the value to be initialised
        void operator ()(value_type &v) const
        {
            P::init(&v);
        }
    };
    /// The copy function object type
    struct _copy
    {
        /// The function call operator, which causes the destination to be copied from the source
        void operator ()(value_type &dest, value_type const &src) const
        {
            P::copy(&dest, &src);
        }
    };
    /// The clear function object type
    struct _clear
    {
        /// The function call operator, which causes the value to be cleared
        void operator ()(value_type &v) const
        {
            P::clear(&v);
        }
    };
};

/// A template for adapting COM enumeration to STL-compatible sequence iteration
///
/// \param I Interface
/// \param V Value type
/// \param VP Value policy type
/// \param R Reference type
/// \param CP Cloning policy type
/// \param Q Quanta
///
/// The various parameterising types are used to stipulate the interface and the
/// value type, and how they are to be handled.
/// 
/// For example, the following parameterisation defines a sequence operating
/// over an <b>IEnumGUID</b> enumerator instance.
///
/// \htmlonly
/// <table border = 0 cellspacing = 0 cellpadding = 0 >
///  <tr><td>&nbsp;&nbsp;</td><td><code>typedef enum_simple_sequence</code></td><td><code>&lt;&nbsp;IEnumGUID</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>,&nbsp;GUID</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>,&nbsp;GUID_policy</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>,&nbsp;GUID const &</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>,&nbsp;forward_cloning_policy&lt;IEnumGUID&gt;</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>,&nbsp;5</code></td></tr>
///  <tr><td>&nbsp;&nbsp;</td><td><code>&nbsp;</code></td><td><code>&gt;&nbsp;&nbsp;&nbsp;enum_sequence_t;</code></td></tr>
/// </table>
/// \endhtmlonly
///
/// The value type is <b>GUID</b> and it is returned as a reference, as
/// the <b>GUID const &</b> in fact.
///
/// The \ref COMSTL type <b>GUID_policy</b> controls how the <b>GUID</b>
/// instances are initialised, copied and destroyed.
///
/// The \ref COMSTL type forward_cloning_policy allows the sequence to provide
/// <a href = "http://sgi.com/tech/stl/ForwardIterator.html">Forward Iterator</a>
/// semantics.
///
/// And the <b>5</b> indicates that the sequence should grab 5 values at a time,
/// to save round trips to the enumerator.
///
/// So this would be used like the following:
///
/// \htmlonly
/// <code>
/// &nbsp;&nbsp;void dump_GUID(GUID const &);
/// <br>
///
/// <br>
/// &nbsp;&nbsp;LPENUMGUID&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;*penGUIDs = . . .;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// Create an instance from wherever
/// <br>
/// &nbsp;&nbsp;enum_sequence_t&nbsp;&nbsp;&nbsp;guids(penGUIDs, false); // Eat the reference
/// <br>
///
/// <br>
/// &nbsp;&nbsp;std::for_each(guids.begin(), guids.end(), dump_GUID);
/// <br>
/// </code>
/// \endhtmlonly
///
template<   cs_typename_param_k I   /* Interface */
        ,   cs_typename_param_k V   /* Value type */
        ,   cs_typename_param_k VP  /* Value policy type */
        ,   cs_typename_param_k R   /* Reference type */
        ,   cs_typename_param_k CP  /* Cloning policy type */
        ,   cs_size_t           Q   /* Quanta */
        >
class enum_simple_sequence
{
public:
    /// Interface type
    typedef I                                           interface_type;
    /// Value type
    typedef V                                           value_type;
    /// Value policy type
    typedef policy_adaptor<VP>                          value_policy_type;
    /// Reference type
    typedef R                                           reference_type;
    /// Cloning policy type
    typedef CP                                          cloning_policy_type;
    enum                                                { quanta = Q };
    /// Type of the current parameterisation
    typedef enum_simple_sequence<I, V, VP, R, CP, Q>    class_type;

public:
    /// Constructor
    ///
    /// \param i The enumeration interface pointer to adapt
    /// \param bAddRef Causes a reference to be added if \c true, otherwise the sequence is deemed to <i>sink</i>, or consume, the interface pointer
    enum_simple_sequence(interface_type *i, cs_bool_t bAddRef)
        : m_i(i)
    {
        if(bAddRef)
        {
            m_i->AddRef();
        }
    }
    /// Releases the adapted interface pointer
    ~enum_simple_sequence()
    {
        m_i->Release();
    }

public:
    /// Iterator for the enum_simple_sequence class
    class iterator
    {
        typedef iterator                                    class_type;
        typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;
    public:
        /// Constructor
        iterator(interface_type *i)
            : m_i(i)
            , m_acquired(0)
            , m_current(0)
        {
            // Default initialise all elements
            comstl_ns_qual_std(for_each)(&m_values[0], &m_values[quanta], value_policy_type::_init());

            m_i->AddRef();

            operator ++();
        }
        /// Default constructor
        iterator()
            : m_i(0)
            , m_acquired(0)
            , m_current(0)
        {
            // Default initialise all elements
            comstl_ns_qual_std(for_each)(&m_values[0], &m_values[quanta], value_policy_type::_init());
        }
        /// <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">Move constructor</a>
#ifdef __STLSOFT_COMPILER_IS_MSVC
        iterator(class_type const &rhs)
            : m_i(cloning_policy_type::clone(const_cast<interface_type *&>(rhs.m_i)))
#else
        iterator(rhs_type rhs)
            : m_i(cloning_policy_type::clone(move_lhs_from_rhs(rhs).m_i))
#endif /* __STLSOFT_COMPILER_IS_MSVC */
            , m_acquired(rhs.m_acquired)
            , m_current(rhs.m_current)
        {
            value_type          *begin      =   &m_values[0];
            value_type          *end        =   &m_values[quanta];
            value_type const    *src_begin  =   &rhs.m_values[0];
            value_type const    *src_end    =   &rhs.m_values[rhs.m_acquired];

            for(; src_begin != src_end; ++begin, ++src_begin)
            {
                value_policy_type::init(begin);
                value_policy_type::copy(begin, src_begin);
            }

            for(; begin != end; ++begin)
            {
                value_policy_type::init(begin);
            }
        }
        /// Releases any internal storage
        ~iterator()
        {
            // Clear all elements
            comstl_ns_qual_std(for_each)(&m_values[0], &m_values[m_acquired], value_policy_type::_clear());

            safe_release(m_i);
        }

    public:
        /// Pre-increment operator
        iterator &operator ++()
        {
            if(++m_current < m_acquired)
            {
                // Do nothing
            }
            else if(NULL == m_i)
            {
                comstl_message_assert("Attempting to increment an invalid iterator", 0 < m_acquired && m_acquired < quanta && m_current == m_acquired);

                m_acquired = 0;
            }
            else
            {
                comstl_message_assert("Attempting to increment an invalid iterator", NULL != m_i);

                // Clear all elements
                comstl_ns_qual_std(for_each)(&m_values[0], &m_values[m_acquired], value_policy_type::_clear());

                // Reset enumeration
                m_current = 0;

                ULONG   cFetched    =   0;
                HRESULT hr          =   m_i->Next(quanta, &m_values[0], &cFetched);

                // We no longer checked for a FAILED(hr), since some iterators
                // return invalid results. We rely on cFetched, which is the
                // only reliable guide when marshalling anyway

                m_acquired = cFetched;

                if( hr == S_FALSE ||
                    m_acquired == 0)
                {
                    release_set_null(m_i);
                }
            }

            return *this;
        }

        void operator ++(int)
        {
            operator ++();
        }

    public:
        /// Returns the value represented by the current iteration position
        reference_type operator *()
        {
            return m_values[m_current];
        }

    public:
        /// Evaluates whether \c this and \c rhs are equivalent
        cs_bool_t operator == (iterator const &rhs) const
        {
            // The only valid comparison is when they both represent the end
            // values.

            return m_acquired == 0 && m_i == 0 && rhs.m_i == 0;
        }
        /// Evaluates whether \c this and \c rhs are not equivalent
        cs_bool_t operator != (iterator const &rhs) const
        {
            return !operator == (rhs);
        }

    private:
        interface_type  *m_i;
        cs_size_t       m_acquired;
        cs_size_t       m_current;
        value_type      m_values[quanta];
    };

public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    iterator begin() const
    {
        m_i->Reset();

        return iterator(m_i);
    }
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    iterator end() const
    {
        return iterator();
    }

// Members
private:
    interface_type  *m_i;

// Implementation
private:
    enum_simple_sequence(class_type const &);
    enum_simple_sequence const &operator =(class_type const &);
};


/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_itf_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace stlsoft::comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_ENUM_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
