/* ////////////////////////////////////////////////////////////////////////////
 * File:        comstl_interface_cast.h
 *
 * Purpose:     Safe interface casting functions.
 *
 * Created:
 * Updated:     11th August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_INTERFACE_CAST
#define _COMSTL_INCL_H_COMSTL_INTERFACE_CAST

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_INTERFACE_CAST_MAJOR       1
#define _COMSTL_VER_H_COMSTL_INTERFACE_CAST_MINOR       6
#define _COMSTL_VER_H_COMSTL_INTERFACE_CAST_REVISION    4
#define _COMSTL_VER_H_COMSTL_INTERFACE_CAST_EDIT        39
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                    // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */
#ifndef _COMSTL_INCL_H_COMSTL_REFCOUNT_FUNCTIONS
 #include "comstl_refcount_functions.h" // safe_addref, safe_release
#endif /* !_COMSTL_INCL_H_COMSTL_REFCOUNT_FUNCTIONS */
#ifndef _COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS
 #include "comstl_interface_traits.h"   // Include IID_traits
#endif /* !_COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup comstl_itf_library Interface Library
/// \ingroup COMSTL libraries
/// \brief This library provides facilities for manipulating COM interfaces
/// @{

/* ////////////////////////////////////////////////////////////////////////////
 * Functions
 */

// This helper converts from an interface pointer to itself.
//
// It explicitly takes and returns pointer so that it disambiguates from any
// overload that takes an interface wrapper instance by value/reference.
template <cs_typename_param_k I>
inline I *simple_interface_cast(I *pi)
{
    return pi;
}

/* ////////////////////////////////////////////////////////////////////////////
 * Functionals
 */

/// A null exception generating type
struct ignore_interface_cast_exception
{
public:
    /// The null exception type
    struct thrown_type
    {
    };

public:
    /// The function call operator, which does not throw an exception
    ///
    /// \param hr The HRESULT that caused the error
    /// \param riid The REFIID that could not be acquired
    void operator ()(HRESULT /* hr */, REFIID /*riid*/) comstl_throw_0()
    {
        // Do nothing
    }
};

/// A function object that calls Release() on the interface
template <cs_typename_param_k I>
struct noaddref_release
{
public:
    /// The function call operator, that calls Release() on the interface
    ///
    /// \param pi The interface pointer
    void operator ()(I pi)
    {
        release(pi);
    }
};

/// A function object that does not call Release() on the interface
template <cs_typename_param_k I>
struct addref_release
{
public:
    /// The function call operator, which does not call Release() on the interface
    ///
    /// \param pi The interface pointer
    void operator ()(I /* pi */)
    {
    }
};

/* /////////////////////////////////////////////////////////////////////////////
 * Raw-pointer safety
 *
 * Alas this requires partial template specialisation, so is not available in
 * all environments.
 */

#ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

/// Interface pointer traits
///
/// \param I The interface type
template <typename I>
struct interface_pointer_traits;

/// A specialisation for pointers only
template <typename I>
struct interface_pointer_traits<I*>
{
    typedef I   interface_type;
};

/// A veneer interface that hides the AddRef() and Release() methods
///
/// \param I The interface to wrap
template <cs_typename_param_k I>
interface protect_refcount
    : public I
{
private:
    STDMETHOD_(ULONG, AddRef)()
    {
        I   *pi = static_cast<I*>(this);

        return pi->AddRef();
    }
    STDMETHOD_(ULONG, Release)()
    {
        I   *pi = static_cast<I*>(this);

        return pi->Release();
    }
};

#endif /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */

/* ////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Base class for the interface cast classes
///
/// This class serves only as a base, and cannot be used in isolation
///
/// \param I The interface pointer type
/// \param R The release type
/// \param X The exception type
template<   cs_typename_param_k I
        ,   cs_typename_param_k R
        ,   cs_typename_param_k X
        >
class interface_cast_base
{
public:
    /// The interface pointer type
    typedef I                                                               interface_pointer_type;
#ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
    /// The interface type
    typedef cs_typename_type_k interface_pointer_traits<I>::interface_type  interface_type;
#endif /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */
    /// The release type
    typedef R                                                               release_type;
    /// The exception type
    typedef X                                                               exception_type;
    /// The thrown type
    typedef cs_typename_type_k exception_type::thrown_type                  thrown_type;
    /// The type of the current parameterisation
    typedef interface_cast_base<I, R, X>                                    class_type;

// Construction
protected:
    /// Constructor that attempts the speculative cast
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <cs_typename_param_k J>
    cs_explicit_k interface_cast_base(J &j)
        : m_pi(do_cast(simple_interface_cast(j)))
    {}
#else
    cs_explicit_k interface_cast_base(LPUNKNOWN punk)
        : m_pi(do_cast(punk))
    {}
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */

    /// Constructor that directly casts (without calling QueryInterface())
    cs_explicit_k interface_cast_base(interface_pointer_type pi)
        : m_pi(pi)
    {
        addref(m_pi);
    }

protected:
    /// Releases the acquired interface pointer according to the \c release_type policy
    ~interface_cast_base() comstl_throw_0()
    {
        if(NULL != m_pi)
        {
            release_type()(m_pi);
        }
    }

// Implementation
protected:
    /// Perform the cast, throwing the \c exception_type's \c thrown_type if the
    /// requested interface cannot be acquired.
    ///
    /// \param punk The interface pointer to cast
    /// \return The converted interface pointer
    static interface_pointer_type do_cast(LPUNKNOWN punk) comstl_throw_1 (thrown_type)
    {
        interface_pointer_type  pi;

        if(NULL == punk)
        {
            pi = NULL;
        }
        else
        {
            REFIID  iid =   IID_traits<interface_pointer_type>().iid();
            HRESULT hr  =   punk->QueryInterface(iid, reinterpret_cast<void**>(&pi));

            if(FAILED(hr))
            {
                exception_type()(hr, iid);

                pi = NULL;
            }
        }

        return pi;
    }

    /// Returns a non-mutating reference to the acquired interface pointer
    interface_pointer_type const &_get_pointer()
    {
        return m_pi;
    }

    /// Returns a copy of the acquired interface pointer
    interface_pointer_type _get_pointer() const
    {
        return m_pi;
    }

// Members
private:
    interface_pointer_type const    m_pi;

// Not to be implemented
public:
    interface_cast_base(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);
};


/// Interface cast that does not add a net reference count
///
/// This class provides a cast between interface pointers, but does not add a net reference count
///
/// \param I The interface pointer type
/// \param X The exception type
template<   cs_typename_param_k I
        ,   cs_typename_param_k X
        >
class interface_cast_noaddref
    : protected interface_cast_base<I, noaddref_release<I>, X>
{
private:
    typedef interface_cast_base<I, noaddref_release<I>, X>                  parent_class_type;
public:
    /// The type of the current parameterisation
    typedef interface_cast_noaddref<I, X>                                   class_type;
    /// The interface pointer type
    typedef cs_typename_type_k parent_class_type::interface_pointer_type    interface_pointer_type;
#ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
    /// The interface type
    typedef cs_typename_type_k parent_class_type::interface_type            interface_type;
    typedef protect_refcount<interface_type>                                *protected_interface_pointer_type;
#else
    typedef interface_pointer_type                                          protected_interface_pointer_type;
#endif // __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

// Construction
public:
    /// Constructor that attempts the speculative cast
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <cs_typename_param_k J>
    cs_explicit_k interface_cast_noaddref(J &j)
        : parent_class_type(j)
    {}
#else
    cs_explicit_k interface_cast_noaddref(LPUNKNOWN punk)
        : parent_class_type(punk)
    {}
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */

    /// Constructor that directly casts (without calling QueryInterface())
    cs_explicit_k interface_cast_noaddref(interface_pointer_type pi)
        : parent_class_type(pi)
    {}

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
    ~interface_cast_noaddref() comstl_throw_0()
    {} // We need to provide this to persuade VC6 to call the parent class dtor
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

// State
public:
    /// Represents whether the cast succeeded
    ///
    /// \retval true The interface cast succeeded
    /// \retval false The interface cast failed
    operator cs_bool_t () const
    {
        return _get_pointer() != NULL;
    }

    /// Represents whether the cast failed
    ///
    /// \retval true The interface cast failed
    /// \retval false The interface cast succeeded
    cs_bool_t operator ! () const
    {
        return _get_pointer() == NULL;
    }

// Accessors
public:
    /// Access the members of the interface
    protected_interface_pointer_type operator -> () const
    {
        comstl_message_assert("Attempting to dereference null pointer. Exception model should not be null", _get_pointer() != NULL);

        return static_cast<protected_interface_pointer_type>(_get_pointer());
    }

// Not to be implemented
private:
    interface_cast_noaddref(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);

#ifdef __STLSOFT_CF_NATIVE_BOOL_SUPPORT
    operator interface_pointer_type () const;
#endif /* __STLSOFT_CF_NATIVE_BOOL_SUPPORT */
//    interface_pointer_type operator -> ();

    // These are defined to placate Borland C/C++
    void *operator new(cs_size_t /* si */) { return 0; }
    void operator delete(void * /* pv */) {}
};

/// Interface cast that does add a net reference count
///
/// This class provides a cast between interface pointers, that adds a net reference count
///
/// \param I The interface pointer type
/// \param X The exception type, defaulted to ignore_interface_cast_exception
template<   cs_typename_param_k I
        ,   cs_typename_param_k X = ignore_interface_cast_exception
        >
class interface_cast_addref
    : protected interface_cast_base<I, addref_release<I>, X>
{
private:
    typedef interface_cast_base<I, addref_release<I>, X>                    parent_class_type;
public:
    /// The type of the current parameterisation
    typedef interface_cast_addref<I, X>                                     class_type;
    /// The interface pointer type
    typedef cs_typename_type_k parent_class_type::interface_pointer_type    interface_pointer_type;
#ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
    /// The interface type
    typedef cs_typename_type_k parent_class_type::interface_type            interface_type;
    typedef protect_refcount<interface_type>                                *protected_interface_pointer_type;
#else
    typedef interface_pointer_type                                          protected_interface_pointer_type;
#endif // __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

// Construction
public:
    /// Constructor that attempts the speculative cast
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <cs_typename_param_k J>
    cs_explicit_k interface_cast_addref(J j)
        : parent_class_type(j)
    {}
#else
    cs_explicit_k interface_cast_addref(LPUNKNOWN punk)
        : parent_class_type(punk)
    {}
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */

    /// Constructor that directly casts (without calling QueryInterface())
    cs_explicit_k interface_cast_addref(interface_pointer_type pi)
        : parent_class_type(pi)
    {}

// Accessors
public:
    /// A pointer to the acquired interface
    operator protected_interface_pointer_type ()
    {
        return static_cast<protected_interface_pointer_type>(_get_pointer());
    }

    /// Access the members of the interface
    protected_interface_pointer_type operator -> ()
    {
        comstl_message_assert("Attempting to dereference null pointer. Exception model should not be null", _get_pointer() != NULL);

        return static_cast<protected_interface_pointer_type>(_get_pointer());
    }

// Not to be implemented
private:
    interface_cast_addref(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);

    // These are defined to placate Borland C/C++
    void *operator new(cs_size_t /* si */) { return 0; }
    void operator delete(void * /* pv */) {}
};

/* /////////////////////////////////////////////////////////////////////////////
 * Shims
 */

/// Attribute shim to retrieve the interface pointer of the given cast instance
///
/// \param I The interface type
/// \param X The exception type
/// \return The raw interface pointer
template<   cs_typename_param_k I
        ,   cs_typename_param_k X
        >
inline I get_ptr(interface_cast_noaddref<I, X> &p)
{
    return p.operator -> ();
}

/// Attribute shim to retrieve the interface pointer of the given cast instance
///
/// \param I The interface type
/// \param X The exception type
/// \return The raw interface pointer
template<   cs_typename_param_k I
        ,   cs_typename_param_k X
        >
inline I const get_ptr(interface_cast_noaddref<I, X> const &p)
{
    return p.operator -> ();
}

/// Attribute shim to retrieve the interface pointer of the given cast instance
///
/// \param I The interface type
/// \param X The exception type
/// \return The raw interface pointer
template<   cs_typename_param_k I
        ,   cs_typename_param_k X
        >
inline I get_ptr(interface_cast_addref<I, X> &p)
{
    return p;
}

/// Attribute shim to retrieve the interface pointer of the given cast instance
///
/// \param I The interface type
/// \param X The exception type
/// \return The raw interface pointer
template<   cs_typename_param_k I
        ,   cs_typename_param_k X
        >
inline I const get_ptr(interface_cast_addref<I, X> const &p)
{
    return p;
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_itf_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_INTERFACE_CAST */

/* ////////////////////////////////////////////////////////////////////////// */
