/* /////////////////////////////////////////////////////////////////////////////
 * File:        comstl_string_access.h
 *
 * Purpose:     Contains classes and functions for dealing with OLE/COM strings.
 *
 * Created:
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_STRING_ACCESS
#define _COMSTL_INCL_H_COMSTL_STRING_ACCESS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_STRING_ACCESS_MAJOR     1
#define _COMSTL_VER_H_COMSTL_STRING_ACCESS_MINOR     6
#define _COMSTL_VER_H_COMSTL_STRING_ACCESS_REVISION  4
#define _COMSTL_VER_H_COMSTL_STRING_ACCESS_EDIT      44
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"    // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_string_access String Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types

/// \defgroup comstl_string_access_shims String Access Shims (COMSTL)
/// \ingroup COMSTL concepts_shims_string_access
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Constants
 */

const cs_size_t COMSTL_CCH_GUID =   38;

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

inline cs_size_t guid2string_w(GUID const &guid, cs_char_w_t buff[1 + COMSTL_CCH_GUID])
{
    return static_cast<cs_size_t>(StringFromGUID2(guid, buff, 1 + COMSTL_CCH_GUID));
}

inline cs_size_t guid2string_a(GUID const &guid, cs_char_a_t buff[1 + COMSTL_CCH_GUID])
{
    const cs_size_t COMSTL_CCH_GUID_AND_NULL    =   COMSTL_CCH_GUID + 1;

    /* Don't ask! */
#ifdef __STLSOFT_COMPILER_IS_BORLAND
    int         __buff[COMSTL_CCH_GUID_AND_NULL];
    cs_char_w_t *_buff  =   (wchar_t *)__buff;
#else
    cs_char_w_t _buff[COMSTL_CCH_GUID_AND_NULL];
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
    cs_size_t   cch =   guid2string_w(guid, _buff);

    ::WideCharToMultiByte(0, 0, _buff, COMSTL_CCH_GUID_AND_NULL, buff, COMSTL_CCH_GUID_AND_NULL, 0, 0);

    return cch;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/* GUID */

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the text of a given GUID
/// may be accessed as a null-terminated string.
template <cs_typename_param_k C>
class c_str_ptr_GUID_proxy
{
public:
    typedef C                       char_type;
    typedef c_str_ptr_GUID_proxy<C> class_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given GUID instance
    ///
    /// \param w The GUID instance from which the text will be retrieved
    cs_explicit_k c_str_ptr_GUID_proxy(GUID const &guid);

    /// Copy constructor
    c_str_ptr_GUID_proxy(class_type const &rhs)
    {
        for(int i = 0; i < comstl_num_elements(m_buffer); ++i)
        {
            m_buffer[i] = rhs.m_buffer[i];
        }
    }

// Accessors
public:
    /// Returns a null-terminated string representing the GUID contents
    operator char_type const *() const
    {
        return m_buffer;
    }

// Members
private:
    char_type   m_buffer[1 + COMSTL_CCH_GUID];

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline c_str_ptr_GUID_proxy<cs_char_a_t>::c_str_ptr_GUID_proxy(GUID const &guid)
{
#ifndef __STLSOFT_COMPILER_IS_BORLAND
    stlsoft_static_assert(comstl_num_elements(m_buffer) > COMSTL_CCH_GUID);
#endif /* !__STLSOFT_COMPILER_IS_BORLAND */

    guid2string_a(guid, m_buffer);
}

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline c_str_ptr_GUID_proxy<cs_char_w_t>::c_str_ptr_GUID_proxy(GUID const &guid)
{
#ifndef __STLSOFT_COMPILER_IS_BORLAND
    stlsoft_static_assert(comstl_num_elements(m_buffer) > COMSTL_CCH_GUID);
#endif /* !__STLSOFT_COMPILER_IS_BORLAND */

    guid2string_w(guid, m_buffer);
}


/* VARIANT */

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the text of a given variant
/// may be accessed as a null-terminated string.
class c_str_null_VARIANT_proxy
{
public:
    typedef c_str_null_VARIANT_proxy                    class_type;
    typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given BSTR
    ///
    /// \param s The BSTR from which the text will be retrieved
    cs_explicit_k c_str_null_VARIANT_proxy(const BSTR s)
        : m_bstr(s)
        , m_own(false)
    {}

    /// Constructs an instance of the proxy from the given BSTR
    ///
    /// \param ps Pointer to the BSTR from which the text will be retrieved
    cs_explicit_k c_str_null_VARIANT_proxy(BSTR *ps)
        : m_bstr(*ps)
        , m_own(true)
    {
        if(m_own)
        {
            *ps = NULL;
        }
    }

    /// Default constructor
    c_str_null_VARIANT_proxy()
        : m_bstr(NULL)
        , m_own(false)
    {}

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_null_VARIANT_proxy(rhs_type rhs)
        : m_bstr(rhs.m_bstr)
        , m_own(rhs.m_own)
    {
        move_lhs_from_rhs(rhs).m_bstr  =   NULL;
        move_lhs_from_rhs(rhs).m_own   =   false;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_null_VARIANT_proxy()
    {
        if(m_own)
        {
            ::SysFreeString(m_bstr);
        }
    }

// Accessors
public:
    /// Returns a null-terminated string representing the VARIANT contents, or
    /// NULL if the VARIANT contents cannot be converted to text.
    operator LPCOLESTR () const
    {
        return m_bstr;
    }

// Members
private:
    BSTR        m_bstr;
    cs_bool_t   m_own;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_w() function, such that the text of a given variant
/// may be accessed as a null-terminated string.
class c_str_VARIANT_proxy_w
{
public:
    typedef c_str_VARIANT_proxy_w                       class_type;
    typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given BSTR
    ///
    /// \param s The BSTR from which the text will be retrieved
    cs_explicit_k c_str_VARIANT_proxy_w(BSTR &s)
        : m_bstr(s)
    {
        s = NULL;
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_VARIANT_proxy_w(rhs_type rhs)
        : m_bstr(rhs.m_bstr)
    {
        move_lhs_from_rhs(rhs).m_bstr = NULL;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_VARIANT_proxy_w()
    {
        ::SysFreeString(m_bstr);
    }

// Accessors
public:
    /// Returns a null-terminated string representing the VARIANT contents.
    operator LPCOLESTR () const
    {
        return (m_bstr == NULL) ? L"" : m_bstr;
    }

// Members
private:
    BSTR    m_bstr;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_a() function, such that the text of a given variant
/// may be accessed as a null-terminated string.
class c_str_VARIANT_proxy_a
{
public:
    typedef c_str_VARIANT_proxy_a                               class_type;
    typedef stlsoft_define_move_rhs_type(class_type)            rhs_type;
    typedef stlsoft_define_move_rhs_type(c_str_VARIANT_proxy_w) rhs_w_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given c_str_VARIANT_proxy_w
    ///
    /// \param rhs The c_str_VARIANT_proxy_w from which the text will be retrieved
    cs_explicit_k c_str_VARIANT_proxy_a(rhs_w_type rhs)
        : m_proxyw(rhs)
        , m_buffer(0)
    {}

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_VARIANT_proxy_a(rhs_type rhs)
        : m_proxyw(rhs.m_proxyw)
        , m_buffer(rhs.m_buffer)
    {
        move_lhs_from_rhs(rhs).m_buffer = _empty_string();
    }

    /// Releases any storage aquired by the proxy
    ~c_str_VARIANT_proxy_a()
    {
        if(m_buffer != _empty_string())
        {
            ::CoTaskMemFree(m_buffer);
        }
    }

// Accessors
public:
    /// Returns a null-terminated string representing the VARIANT contents.
    operator cs_char_a_t const *() const
    {
        if(m_buffer == 0)
        {
            LPCOLESTR   w_value     =   m_proxyw;
            cs_char_a_t *&_buffer   =   const_cast<class_type*>(this)->m_buffer;

            if( w_value == 0 ||
                *w_value == L'\0')
            {
                _buffer = _empty_string();
            }
            else
            {
                cs_size_t   cch =   ::SysStringLen((BSTR)w_value);

                _buffer = static_cast<cs_char_a_t *>(::CoTaskMemAlloc((1 + cch) * sizeof(cs_char_a_t)));

                if(_buffer == 0)
                {
                    _buffer = _empty_string();
                }
                else
                {
#ifdef WIN32
                    ::WideCharToMultiByte(0, 0, w_value, -1, _buffer, cch + 1, NULL, NULL);
#else
#error Not currently implemented for operating systems other than Win32
#endif /* __SYNSOFT_DBS_DEBUG */
                }
            }
        }

        return m_buffer;
    }

// Implementation
private:
    static cs_char_a_t *_empty_string()
    {
        static cs_char_a_t  s_empty[1] = "";

        return s_empty;
    }

// Members
private:
    c_str_VARIANT_proxy_w   m_proxyw;
    cs_char_a_t             *m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr_null
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or NULL.
 */

/* GUID */
/// \brief Returns the corresponding ANSI C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_a_t> c_str_ptr_null_a(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_a_t>(guid);
}

/// \brief Returns the corresponding Unicode C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_w_t> c_str_ptr_null_w(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_w_t>(guid);
}

/// \brief Returns the corresponding C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_o_t> c_str_ptr_null_o(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_o_t>(guid);
}

/// \brief Returns the corresponding C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_o_t> c_str_ptr_null(GUID const &guid)
{
    return c_str_ptr_null_o(guid);
}

/* VARIANT */
/// \brief Returns the corresponding ANSI C-string pointer of the VARIANT \c v, or a null pointer
inline c_str_null_VARIANT_proxy c_str_ptr_null(VARIANT const &v)
{
    if(v.vt == VT_BSTR)
    {
        return c_str_null_VARIANT_proxy(v.bstrVal);
    }
    else if(v.vt == VT_NULL ||
            v.vt == VT_EMPTY)
    {
        return c_str_null_VARIANT_proxy();
    }
    else
    {
        VARIANT vs;
        HRESULT hr;

        ::VariantInit(&vs);

        hr  =   ::VariantChangeTypeEx(&vs, const_cast<VARIANT *>(&v), LOCALE_USER_DEFAULT, 0, VT_BSTR);

        if(FAILED(hr))
        {
            vs.bstrVal = NULL;
        }

        return c_str_null_VARIANT_proxy(&vs.bstrVal);
    }
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or to an empty string.
 */

/* GUID */
/// \brief Returns the corresponding ANSI C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_a_t> c_str_ptr_a(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_a_t>(guid);
}

/// \brief Returns the corresponding Unicode C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_w_t> c_str_ptr_w(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_w_t>(guid);
}

/// \brief Returns the corresponding C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_o_t> c_str_ptr_o(GUID const &guid)
{
    return c_str_ptr_GUID_proxy<cs_char_o_t>(guid);
}

/// \brief Returns the corresponding C-string pointer of the GUID \c guid
inline c_str_ptr_GUID_proxy<cs_char_o_t> c_str_ptr(GUID const &guid)
{
    return c_str_ptr_o(guid);
}

/* VARIANT */
/// \brief Returns the corresponding ANSI C-string pointer of the VARIANT \c v
inline c_str_VARIANT_proxy_a c_str_ptr_a(VARIANT const &v)
{
    VARIANT vs;
    HRESULT hr;

    ::VariantInit(&vs);

    hr  =   ::VariantChangeTypeEx(&vs, const_cast<VARIANT *>(&v), LOCALE_USER_DEFAULT, 0, VT_BSTR);

    if(FAILED(hr))
    {
        vs.bstrVal = NULL;
    }

    return c_str_VARIANT_proxy_a(c_str_VARIANT_proxy_w(vs.bstrVal));
}

/// \brief Returns the corresponding Unicode C-string pointer of the VARIANT \c v
inline c_str_VARIANT_proxy_w c_str_ptr_w(VARIANT const &v)
{
    VARIANT vs;
    HRESULT hr;

    ::VariantInit(&vs);

    hr  =   ::VariantChangeTypeEx(&vs, const_cast<VARIANT *>(&v), LOCALE_USER_DEFAULT, VARIANT_ALPHABOOL, VT_BSTR);

    if(FAILED(hr))
    {
        vs.bstrVal = NULL;
    }

    return c_str_VARIANT_proxy_w(vs.bstrVal);
}

/// \brief Returns the corresponding C-string pointer of the VARIANT \c v
inline c_str_VARIANT_proxy_w c_str_ptr(VARIANT const &v)
{
    return c_str_ptr_w(v);
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_len
 *
 * This can be applied to an expression, and the return value is the number of
 * characters in the character string in the expression.
 */

/* GUID */
/// \brief Returns the length (in characters) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len_a(GUID const &/* guid */)
{
    return COMSTL_CCH_GUID;
}

/// \brief Returns the length (in characters) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len_w(GUID const &/* guid */)
{
    return COMSTL_CCH_GUID;
}

/// \brief Returns the length (in characters) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len_o(GUID const &/* guid */)
{
    return COMSTL_CCH_GUID;
}

/// \brief Returns the length (in characters) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len(GUID const &/* guid */)
{
    return COMSTL_CCH_GUID;
}

/* VARIANT */
/// \brief Returns the length (in characters) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len(VARIANT const &v)
{
    cs_size_t   len;

    if(v.vt == VT_BSTR)
    {
        len = v.bstrVal != NULL ? ::SysStringLen(v.bstrVal) : 0;
    }
    else if(v.vt == VT_NULL ||
            v.vt == VT_EMPTY)
    {
        len = 0;
    }
    else
    {
        VARIANT     vs;
        HRESULT     hr;

        ::VariantInit(&vs);

        hr  =   ::VariantChangeTypeEx(&vs, const_cast<VARIANT *>(&v), LOCALE_USER_DEFAULT, 0, VT_BSTR);

        if(FAILED(hr))
        {
            len = 0;
        }
        else
        {
            len = vs.bstrVal ? ::SysStringLen(vs.bstrVal) : 0;

            ::VariantClear(&vs);
        }
    }

    return len;
}

/// \brief Returns the length (in characters) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len_a(VARIANT const &v)
{
    return c_str_len(v);
}

/// \brief Returns the length (in characters) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_len_w(VARIANT const &v)
{
    return c_str_len(v);
}

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_size
 *
 * This can be applied to an expression, and the return value is the number of
 * bytes required to store the character string in the expression, NOT including
 * the null-terminating character.
 */

/* GUID */
/// \brief Returns the size (in bytes) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size_a(GUID const &guid)
{
    return c_str_len_a(guid) *sizeof(cs_char_a_t);
}

/// \brief Returns the size (in bytes) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size_w(GUID const &guid)
{
    return c_str_len_w(guid) * sizeof(cs_char_w_t);
}

/// \brief Returns the size (in bytes) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size_o(GUID const &guid)
{
    return c_str_len_o(guid) * sizeof(cs_char_o_t);
}

/// \brief Returns the size (in bytes) of the GUID \c guid, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size(GUID const &guid)
{
    return c_str_size_o(guid);
}

/* VARIANT */
/// \brief Returns the size (in bytes) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size_a(VARIANT const &v)
{
    return c_str_len(v) * sizeof(cs_char_a_t);
}

/// \brief Returns the size (in bytes) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size_w(VARIANT const &v)
{
    return c_str_len(v) * sizeof(OLECHAR);
}

/// \brief Returns the size (in bytes) of the VARIANT \c v, <b><i>not</i></b> including the null-terminating character
inline cs_size_t c_str_size(VARIANT const &v)
{
    return c_str_size_w(v);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_string_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace stlsoft::comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_STRING_ACCESS */

/* ////////////////////////////////////////////////////////////////////////// */
