/* ////////////////////////////////////////////////////////////////////////////
 * File:        mfcstl_grab_cstring_buffer.h (Formerly MFGrbStr.h: ::SynesisMfc)
 *
 * Purpose:     CString Get/ReleaseBuffer scoping class.
 *
 * Created:     12th February 1999
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/mfcstl
 *                          http://www.mfcstl.org/
 *
 *              email:      submissions@mfcstl.org  for submissions
 *                          admin@mfcstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _MFCSTL_INCL_H_MFCSTL_GRAB_CSTRING_BUFFER
#define _MFCSTL_INCL_H_MFCSTL_GRAB_CSTRING_BUFFER

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _MFCSTL_VER_H_MFCSTL_GRAB_CSTRING_BUFFER_MAJOR      1
#define _MFCSTL_VER_H_MFCSTL_GRAB_CSTRING_BUFFER_MINOR      2
#define _MFCSTL_VER_H_MFCSTL_GRAB_CSTRING_BUFFER_REVISION   2
#define _MFCSTL_VER_H_MFCSTL_GRAB_CSTRING_BUFFER_EDIT       22
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _MFCSTL_INCL_H_MFCSTL
#include <mfcstl.h> // Include the MFCSTL root header
#endif /* !_MFCSTL_INCL_H_MFCSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The MFCSTL components are contained within the mfcstl namespace. This is
 * actually an alias for stlsoft::mfcstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _MFCSTL_NO_NAMESPACE    mfcstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::mfcstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             mfcstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _MFCSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::mfcstl */
namespace mfcstl
{
 #else
/* Define stlsoft::mfcstl_project */

namespace stlsoft
{

namespace mfcstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_string String Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating strings

/// \defgroup mfcstl_string_library String Library (MFCSTL)
/// \ingroup MFCSTL libraries_string
/// \brief This library provides facilities for defining and manipulating MFC string types
/// @{

/* ////////////////////////////////////////////////////////////////////////////
 * grab_cstring_buffer
 *
 * This class is used to scope the aquisition and release of a CString buffer
 * ensuring that ReleaseBuffer() is called on the given CString object.
 */

/// \brief Scopes the aquisition and release of a CString buffer
///
/// This class is used to scope the aquisition and release of a CString buffer
/// ensuring that ReleaseBuffer() is called on the given CString object.
class grab_cstring_buffer
{
private:
    typedef grab_cstring_buffer   class_type;

// Construction
public:
    /// \brief Acquires the requested length on the given managed string
    ///
    /// Calls \c GetBuffer(length) on the given string, after recording the 
    /// original length. If acquisition fails then a CMemoryException is thrown.
    grab_cstring_buffer(CString &str, int length) mfcstl_throw_1(CMemoryException *);
    /// Releases the managed string.
    ~grab_cstring_buffer() mfcstl_throw_0();

// Conversion operators
public:
    /// Provides mutating access to the managed string's internal buffer
    operator LPTSTR();
    /// Provides non-mutating access to the managed string's internal buffer
    operator LPCTSTR() const;

// Attributes
public:
    /// Returns the length of the managed string
    int length() const;
    /// Returns the original length of the managed string
    int original_length() const;

// Members
private:
    CString         &m_str;
    const int       m_len;
    const int       m_originalLen;
    const LPTSTR    m_psz;

// Not implemented
private:
    grab_cstring_buffer(class_type const &rhs);
    const grab_cstring_buffer &operator =(class_type const &rhs);
};

///////////////////////////////////////////////////////////////////////////////
// Implementation

inline grab_cstring_buffer::grab_cstring_buffer(CString &str, int length) mfcstl_throw_1(CMemoryException *)
    : m_str(str)
    , m_len(length)
    , m_originalLen(str.GetLength())
    , m_psz(str.GetBuffer(length))
{
    // Fortunately, objects of this class contain no resources of their own, and so
    // throwing an exception from the constructor is somewhat benign.
}

inline grab_cstring_buffer::~grab_cstring_buffer() mfcstl_throw_0()
{
    // Dtor will never be called if GetBufferSetLength throws,
    // so we can assume we have a valid reference to an open
    // string which needs to be released here.
    m_str.ReleaseBuffer();
}

inline grab_cstring_buffer::operator LPTSTR()
{
    return m_psz;
}

inline grab_cstring_buffer::operator LPCTSTR() const
{
    return m_psz;
}

inline int grab_cstring_buffer::length() const
{
    return m_len;
}

inline int grab_cstring_buffer::original_length() const
{
    return m_originalLen;
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group mfcstl_string_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _MFCSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace mfcstl
 #else
} // namespace mfcstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_MFCSTL_INCL_H_MFCSTL_GRAB_CSTRING_BUFFER */

/* ////////////////////////////////////////////////////////////////////////// */
