/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_char_traits.h (originally MTCTrait.h, ::SynesisStl)
 *
 * Purpose:     char_traits classes.
 *
 * Created:     19th November 1998
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_CHAR_TRAITS
#define _STLSOFT_INCL_H_STLSOFT_CHAR_TRAITS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_CHAR_TRAITS_MAJOR    1
#define _STLSOFT_VER_H_STLSOFT_CHAR_TRAITS_MINOR    3
#define _STLSOFT_VER_H_STLSOFT_CHAR_TRAITS_REVISION 4
#define _STLSOFT_VER_H_STLSOFT_CHAR_TRAITS_EDIT     27
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"               // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_SIGN_TRAITS
 #include "stlsoft_sign_traits.h"   // stlsoft::sign_traits
#endif /* !_STLSOFT_INCL_H_STLSOFT_SIGN_TRAITS */
#ifdef _STLSOFT_NO_NAMESPACE
 #include <string>                  // std::char_traits
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Constraints
 */

// class stlsoft_char_traits
/// Character traits
///
/// \param C The character type
/// \param N The integer type
/// \param Z The size type
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k N = ss_int_t
        ,   ss_typename_param_k Z = ss_size_t
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        >
struct stlsoft_char_traits
{
public:
    /// The char type
    typedef C                                   char_type;
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The current parameterisation of the type
    typedef stlsoft_char_traits<C, N, Z>        class_type;
    /// The integer type
    typedef N                                   int_type;
    /// The size type
    typedef Z                                   size_type;
#else
    /// The current parameterisation of the type
    typedef stlsoft_char_traits<C>              class_type;
    /// The integer type
    typedef ss_int_t                            int_type;
    /// The size type
    typedef ss_size_t                           size_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The position type
    typedef ss_streampos_t                      pos_type;
    /// The offset type
    typedef ss_streamoff_t                      off_type;

public:
    /// Assigns \c rhs to \c lhs
    static void assign(char_type &lhs, char_type const &rhs)
    {
        lhs = rhs;
    }

    /// Assigns \c cch characters of value \c c to \c dest
    static char_type *assign(char_type *dest, size_type cch, char_type const &c)
    {
        char_type * ret;

        stlsoft_message_assert("char_traits<X>::assign called with NULL destination", cch == 0 || dest != NULL);

        for(ret = dest; 0 < cch; --cch, ++dest)
        {
            assign(*dest, c);
        }

        return ret;
    }

    /// Evaluates whether \c lhs is equivalent to \c rhs
    static ss_bool_t eq(char_type const &lhs, char_type const &rhs)
    {
        return lhs == rhs;
    }

    /// Evaluates whether \c lhs is less than \c rhs
    static ss_bool_t lt(char_type const &lhs, char_type const &rhs)
    {
        return lhs < rhs;
    }

    /// Compares \c cch characters of \c s1 with \c s2
    ///
    /// \retval <0 s1 is lexicographically less than s2
    /// \retval 0 s1 is lexicographically equal to s2
    /// \retval >0 s1 is lexicographically more than s2
    static int_type compare(char_type const *s1, char_type const *s2, size_type cch)
    {
        stlsoft_message_assert("char_traits<X>::compare called with NULL string", cch == 0 || s1 != NULL);
        stlsoft_message_assert("char_traits<X>::compare called with NULL string", cch == 0 || s2 != NULL);

        for(size_type n = 0; n < cch; ++n, ++s1, ++s2)
        {
            if(!eq(*s1, *s2))
            {
                return lt(*s1, *s2) ? -1 : +1;
            }
        }

        return 0;
    }

    static int_type compare_max(char_type const *s1, char_type const *s2, size_type cch)
    {
        stlsoft_message_assert("char_traits<X>::compare_max called with NULL string", cch == 0 || s1 != NULL);
        stlsoft_message_assert("char_traits<X>::compare_max called with NULL string", cch == 0 || s2 != NULL);

        for(size_type n = 0; n < cch; ++n, ++s1, ++s2)
        {
            if(!eq(*s1, *s2))
            {
                return lt(*s1, *s2) ? -1 : +1;
            }
            else if(eq(*s1, char_type(0)))
            {
                break;
            }
        }

        return 0;
    }

    /// Compares, using compare(), \c s1 with \c s2, either or both of which may be \c null
    static int_type compare_null(char_type const *s1, char_type const *s2, size_type cch)
    {
        int_type    result;

        if( s1 != NULL &&
            s2 != NULL)
        {
            result = compare(s1, s2, cch);
        }
        else
        {
            result = (s1 != NULL) ? +1 : (s2 != NULL) ? -1 : 0;
        }

        return result;
    }

    /// Compares, using compare_max(), \c s1 with \c s2, either or both of which may be \c null
    static int_type compare_maxnull(char_type const *s1, char_type const *s2, size_type cch)
    {
        int_type    result;

        if( s1 != NULL &&
            s2 != NULL)
        {
            result = compare_max(s1, s2, cch);
        }
        else
        {
            result = (s1 != NULL) ? +1 : (s2 != NULL) ? -1 : 0;
        }

        return result;
    }

    /// Evaluates the length of the string \c s
    static size_type length(char_type const *s)
    {
        size_type cch;

        stlsoft_message_assert("char_traits<X>::length called with NULL string", s != NULL);

        for(cch = 0; !eq(*s, char_type(0)); ++s)
        {
            ++cch;
        }

        return cch;
    }

    /// Evaluates the length of the string \c s, which may be \c null
    static size_type length_null(char_type const *s)
    {
        return (s != NULL) ? length(s) : 0;
    }

    /// Evaluates the length of the string \c s up to a given number of characters
    ///
    /// \param s The string to be evaluated. It may <b>not</b> be null
    /// \param s The maximum number of characters to evaluate
    /// \return The length of the string (in characters) not including the null-terminator
    static size_type length_max(char_type const *s, size_type limit)
    {
        size_type cch;

        stlsoft_message_assert("char_traits<X>::length_max called with NULL string", s != NULL);

        for(cch = 0; cch < limit && !eq(*s, char_type(0)); ++s)
        {
            ++cch;
        }

        return cch;
    }

    /// Evaluates the length of the string \c s, which may be \c null, up to a given number of characters
    ///
    /// \param s The string to be evaluated. It may be null
    /// \param s The maximum number of characters to evaluate
    /// \return The length of the string (in characters) not including the null-terminator
    static size_type length_max_null(char_type const *s, size_type limit)
    {
        return (s != NULL) ? length_max(s, limit) : 0;
    }

    /// Copies \c cch characters from \c src to \c dest
    static char_type *copy(char_type *dest, char_type const *src, size_type cch)
    {
        char_type   *ret;

        stlsoft_message_assert("char_traits<X>::copy called with NULL destination", cch == 0 || dest != NULL);
        stlsoft_message_assert("char_traits<X>::copy called with NULL source", cch == 0 || src != NULL);

        for(ret = dest; 0 < cch; --cch, ++dest, ++src)
        {
            assign(*dest, *src);
        }

        return ret;
    }

    /// Copies \c cch characters from \c src to \c dest, accounting for whether the ranges overlap
    static char_type *move(char_type *dest, char_type const *src, size_type cch)
    {
        char_type *const ret = dest;

        stlsoft_message_assert("char_traits<X>::move called with NULL destination", cch == 0 || dest != NULL);
        stlsoft_message_assert("char_traits<X>::move called with NULL source", cch == 0 || src != NULL);

        if( src < dest &&
            dest < src + cch)
        {
            for(dest += cch, src += cch; 0 < cch; --cch)
            {
                assign(*--dest, *--src);
            }
        }
        else
        {
            for(; 0 < cch; --cch, ++dest, ++src)
            {
                assign(*dest, *src);
            }
        }

        return ret;
    }

    /// Finds the first \c c in \c cch elements in \c s, or \c NULL if not found
    static char_type const *find(char_type const *s, size_type cch, char_type const &c)
    {
        stlsoft_message_assert("char_traits<X>::find called with NULL string", cch == 0 || s != NULL);

        for(; 0 < cch; --cch, ++s)
        {
            if(eq(*s, c))
            {
                break;
            }
        }

        return (0 < cch) ? s : NULL;
    }

    /// Represents the character \c in the character type \c char_type
    static char_type to_char_type(int_type const &c)
    {
        return static_cast<char_type>(c);
    }

    /// Represents the character \c in the integer type \c int_type
    static int_type to_int_type(char_type const &c)
    {
#if defined(__STLSOFT_COMPILER_IS_WATCOM)
        return (int_type)(c);
#else
        return static_cast<int_type>(static_cast<ss_typename_type_k sign_traits<char_type>::unsigned_type>(c));
#endif /* __STLSOFT_COMPILER_IS_WATCOM */
    }

    /// Evaluates whether \c lhs and \c rhs are equivalent
    static ss_bool_t eq_int_type(int_type const &lhs, int_type const &rhs)
    {
        return lhs == rhs;
    }

    /// Returns the value representing the end-of-file.
    static int_type eof()
    {
        return static_cast<int_type>(-1);
    }

    /// Evaluates whether the given character is the end-of-file.
    static int_type not_eof(int_type const &c)
    {
        return (c != eof() ? c : !eof());
    }
};

// class stlsoft_char_traits_safe
/// Character traits, all of the operations of which can work with null pointers
///
/// \param C The character type
/// \param N The integer type
/// \param Z The size type
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k N = ss_int_t
        ,   ss_typename_param_k Z = ss_size_t
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        >
struct stlsoft_char_traits_safe
{
public:
    /// The character type
    typedef C                                   char_type;
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The current parameterisation of the type
    typedef stlsoft_char_traits_safe<C, N, Z>   class_type;
    /// The integer type
    typedef N                                   int_type;
    /// The size type
    typedef Z                                   size_type;
#else
    /// The current parameterisation of the type
    typedef stlsoft_char_traits_safe<C>         class_type;
    /// The integer type
    typedef ss_int_t                            int_type;
    /// The size type
    typedef ss_size_t                           size_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The position type
    typedef ss_streampos_t                      pos_type;
    /// The offset type
    typedef ss_streamoff_t                      off_type;

public:
    /// Assigns \c rhs to \c lhs
    static void assign(char_type &lhs, char_type const &rhs)
    {
        lhs = rhs;
    }

    /// Assigns \c cch characters of value \c c to \c dest
    static char_type *assign(char_type *dest, size_type cch, char_type const &c)
    {
        char_type * ret;

        stlsoft_message_assert("char_traits_safe<X>::assign called with NULL destination", dest != NULL);

        for(ret = dest; 0 < cch; --cch, ++dest)
        {
            assign(*dest, c);
        }

        return ret;
    }

    /// Evaluates whether \c lhs is equivalent to \c rhs
    static ss_bool_t eq(char_type const &lhs, char_type const &rhs)
    {
        return lhs == rhs;
    }

    /// Evaluates whether \c lhs is less than \c rhs
    static ss_bool_t lt(char_type const &lhs, char_type const &rhs)
    {
        return lhs < rhs;
    }

    /// Compares \c cch characters of \c s1 with \c s2
    ///
    /// \retval <0 s1 is lexicographically less than s2
    /// \retval 0 s1 is lexicographically equal to s2
    /// \retval >0 s1 is lexicographically more than s2
    static int_type compare(char_type const *s1, char_type const *s2, size_type cch)
    {
        return compare_null(s1, s2, cch);
    }

    static int_type compare_max(char_type const *s1, char_type const *s2, size_type cch)
    {
        return compare_maxnull(s1, s2, cch);
    }

    /// Compares, using compare(), \c s1 with \c s2, either or both of which may be \c null
    static int_type compare_null(char_type const *s1, char_type const *s2, size_type cch)
    {
        int_type    result;

        if( s1 != NULL &&
            s2 != NULL)
        {
            result = compare(s1, s2, cch);
        }
        else
        {
            result = (s1 != NULL) ? +1 : (s2 != NULL) ? -1 : 0;
        }

        return result;
    }

    /// Compares, using compare_max(), \c s1 with \c s2, either or both of which may be \c null
    static int_type compare_maxnull(char_type const *s1, char_type const *s2, size_type cch)
    {
        int_type    result;

        if( s1 != NULL &&
            s2 != NULL)
        {
            result = compare_max(s1, s2, cch);
        }
        else
        {
            result = (s1 != NULL) ? +1 : (s2 != NULL) ? -1 : 0;
        }

        return result;
    }

    /// Evaluates the length of the string \c s up to a given number of characters
    ///
    /// \param s The string to be evaluated. It may be null
    /// \param s The maximum number of characters to evaluate
    /// \return The length of the string (in characters) not including the null-terminator
    static size_type length_max_null(char_type const *s, size_type limit)
    {
        size_type cch = 0;

        if(s != NULL)
        {
            for(cch = 0; cch < limit && !eq(*s, char_type(0)); ++s)
            {
                ++cch;
            }
        }

        return cch;
    }

    /// Evaluates the length of the string \c s, which may be \c null, up to a given number of characters
    ///
    /// \param s The string to be evaluated. It may be null
    /// \param s The maximum number of characters to evaluate
    /// \return The length of the string (in characters) not including the null-terminator
    static size_type length_max(char_type const *s, size_type limit)
    {
        return length_max_null(s, limit);
    }

    /// Evaluates the length of the string \c s, which may be \c null
    static size_type length_null(char_type const *s)
    {
        size_type cch = 0;

        if(s != NULL)
        {
            for(; !eq(*s, char_type(0)); ++s)
            {
                ++cch;
            }
        }

        return cch;
    }

    /// Evaluates the length of the string \c s
    static size_type length(char_type const *s)
    {
        return length_null(s);
    }

    /// Copies \c cch characters from \c src to \c dest
    static char_type *copy(char_type *dest, char_type const *src, size_type cch)
    {
        char_type * ret;

        stlsoft_message_assert("char_traits_safe<X>::copy called with NULL destination", dest != NULL);
        stlsoft_message_assert("char_traits_safe<X>::copy called with NULL source", src != NULL);

        for(ret = dest; 0 < cch; --cch, ++dest, ++src)
        {
            assign(*dest, *src);
        }

        return ret;
    }

    /// Copies \c cch characters from \c src to \c dest, accounting for whether the ranges overlap
    static char_type *move(char_type *dest, char_type const *src, size_type cch)
    {
        char_type const     *ret = dest;

        stlsoft_message_assert("char_traits_safe<X>::move called with NULL destination", dest != NULL);
        stlsoft_message_assert("char_traits_safe<X>::move called with NULL source", src != NULL);

        if( src < dest &&
            dest < src + cch)
        {
            for(dest += cch, src += cch; 0 < cch; --cch)
            {
                assign(*--dest, *--src);
            }
        }
        else
        {
            for(; 0 < cch; --cch, ++dest, ++src)
            {
                assign(*dest, *src);
            }
        }

        return ret;
    }

    /// Finds the first \c c in \c cch elements in \c s, or \c NULL if not found
    static char_type const *find(char_type const *s, size_type cch, char_type const &c)
    {
        if(s != NULL)
        {
            for(; 0 < cch; --cch, ++s)
            {
                if(eq(*s, c))
                {
                    break;
                }
            }
        }

        return (0 < cch) ? s : NULL;
    }

    /// Represents the character \c in the character type \c char_type
    static char_type to_char_type(int_type const &c)
    {
        return static_cast<char_type>(c);
    }

    /// Represents the character \c in the integer type \c int_type
    static int_type to_int_type(char_type const &c)
    {
        return static_cast<int_type>(static_cast<ss_typename_type_k sign_traits<char_type>::unsigned_type>(c));
    }

    /// Evaluates whether \c lhs and \c rhs are equivalent
    static ss_bool_t eq_int_type(int_type const &lhs, int_type const &rhs)
    {
        return lhs == rhs;
    }

    /// Returns the value representing the end-of-file.
    static int_type eof()
    {
        return static_cast<int_type>(-1);
    }

    /// Evaluates whether the given character is the end-of-file.
    static int_type not_eof(int_type const &c)
    {
        return (c != eof() ? c : !eof());
    }
};


#if !defined(_STLSOFT_NO_NAMESPACE) || \
    !defined(__STLSOFT_CF_std_char_traits_AVAILABLE)

// class char_traits
/// Character traits, all of the operations of which can work with null pointers
///
/// \param C The character type
/// \param N The integer type
/// \param Z The size type
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k N = ss_int_t
        ,   ss_typename_param_k Z = ss_size_t
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        >
struct char_traits
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    : public stlsoft_char_traits<C, N, Z>
#else
    : public stlsoft_char_traits<C>
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
{
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    typedef stlsoft_char_traits<C, N, Z>                    parent_class_type;
#else
    typedef stlsoft_char_traits<C>                          parent_class_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
public:
    /// The current parameterisation of the type
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    typedef char_traits<C, N, Z>                            class_type;
#else
    typedef char_traits<C>                                  class_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The character type
    typedef ss_typename_type_k parent_class_type::char_type char_type;
    typedef ss_typename_type_k parent_class_type::int_type  int_type;
    typedef ss_typename_type_k parent_class_type::size_type size_type;
    typedef ss_typename_type_k parent_class_type::pos_type  pos_type;
    typedef ss_typename_type_k parent_class_type::off_type  off_type;
};

// class char_traits_safe
/// Character traits, all of the operations of which can work with null pointers
///
/// \param C The character type
/// \param N The integer type
/// \param Z The size type
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k N = ss_int_t
        ,   ss_typename_param_k Z = ss_size_t
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        >
struct char_traits_safe
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    : public stlsoft_char_traits_safe<C, N, Z>
#else
    : public stlsoft_char_traits_safe<C>
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
{
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    typedef stlsoft_char_traits_safe<C, N, Z>               parent_class_type;
#else
    typedef stlsoft_char_traits_safe<C>                     parent_class_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
public:
    /// The current parameterisation of the type
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    typedef char_traits_safe<C, N, Z>                       class_type;
#else
    typedef char_traits_safe<C>                             class_type;
#endif // __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
    /// The character type
    typedef ss_typename_type_k parent_class_type::char_type char_type;
    typedef ss_typename_type_k parent_class_type::int_type  int_type;
    typedef ss_typename_type_k parent_class_type::size_type size_type;
    typedef ss_typename_type_k parent_class_type::pos_type  pos_type;
    typedef ss_typename_type_k parent_class_type::off_type  off_type;
};

#endif /* !_STLSOFT_NO_NAMESPACE || !__STLSOFT_CF_std_char_traits_AVAILABLE */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_CHAR_TRAITS */

/* ////////////////////////////////////////////////////////////////////////// */
