/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_integer_to_string.h (originally MEL3Str.c, ::SynesisDev)
 *
 * Purpose:     Number to string conversion functions.
 *
 * Created:     7th April 2002
 * Updated:     15th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_INTEGER_TO_STRING
#define _STLSOFT_INCL_H_STLSOFT_INTEGER_TO_STRING

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_INTEGER_TO_STRING_MAJOR       1
#define _STLSOFT_VER_H_STLSOFT_INTEGER_TO_STRING_MINOR       5
#define _STLSOFT_VER_H_STLSOFT_INTEGER_TO_STRING_REVISION    1
#define _STLSOFT_VER_H_STLSOFT_INTEGER_TO_STRING_EDIT        29
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper Functions
 */

/// Provides an indexable sequence of digit characters
template <ss_typename_param_k C>
inline C const *get_digit_character()
{
    static C const  s_characters[19] =
    {
            '9'
        ,   '8'
        ,   '7'
        ,   '6'
        ,   '5'
        ,   '4'
        ,   '3'
        ,   '2'
        ,   '1'
        ,   '0'
        ,   '1'
        ,   '2'
        ,   '3'
        ,   '4'
        ,   '5'
        ,   '6'
        ,   '7'
        ,   '8'
        ,   '9'
    };
    static C const  *s_mid  =   s_characters + 9;

    return s_mid;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/// Highly efficient conversion of unsigned integers to string
///
/// \note The function does not check that cchBuf is sufficient for conversion
/// of the particular integer type, except for a debug version
/// <code>assert()</code>. It is the programmer's responsibility to ensure that 
/// the supplied buffer is large enough for the conversion. This can be done
/// with the use of the printf_traits templates.
///
/// \param buf A buffer of sufficient space within which to convert the integer
/// \param cchBuf The number of characters available in \c buf
/// \param i The integer to convert
template<   ss_typename_param_k C
        ,   ss_typename_param_k I
        >
inline C const *unsigned_integer_to_string(C *buf, ss_size_t cchBuf, I i)
{
    C   *psz    =   buf + cchBuf - 1;  // Set pointer to last character.

    *psz = 0;   // Set the terminating null character.

    do
    {
#if defined(__STLSOFT_COMPILER_IS_MSVC)
        typedef I           rem_t;
#else
        typedef ss_uint_t   rem_t;
#endif /* __STLSOFT_COMPILER_IS_MSVC */

        rem_t   lsd = static_cast<rem_t>(i % 10);   // Determine the least significant digit.

        i /= 10;                                    // Deal with next most significant.

        --psz;                                      // Move back.

        *psz = get_digit_character<C>()[lsd];

    } while(i != 0);

    stlsoft_assert(!(psz < buf));

    return psz;
}


/// Highly efficient conversion of unsigned integers to string
///
/// \note The function does not check that cchBuf is sufficient for conversion
/// of the particular integer type, except for a debug version
/// <code>assert()</code>. It is the programmer's responsibility to ensure that 
/// the supplied buffer is large enough for the conversion. This can be done
/// with the use of the printf_traits templates.
///
/// \param buf A buffer of sufficient space within which to convert the integer
/// \param cchBuf The number of characters available in \c buf
/// \param i The integer to convert
/// \param cchRes Receives the number of characters written
template<   ss_typename_param_k C
        ,   ss_typename_param_k I
        >
inline C const *unsigned_integer_to_string(C *buf, ss_size_t cchBuf, I i, size_t &cchRes)
{
    C const *psz    =   unsigned_integer_to_string<C, I>(buf, cchBuf, i);

    cchRes = cchBuf - (psz - (buf - 1));

    return psz;
}


/// Highly efficient conversion of signed integers to string
///
/// \note The function does not check that cchBuf is sufficient for conversion
/// of the particular integer type, except for a debug version
/// <code>assert()</code>. It is the programmer's responsibility to ensure that 
/// the supplied buffer is large enough for the conversion. This can be done
/// with the use of the printf_traits templates.
///
/// \param buf A buffer of sufficient space within which to convert the integer
/// \param cchBuf The number of characters available in \c buf
/// \param i The integer to convert
template<   ss_typename_param_k C
        ,   ss_typename_param_k I
        >
inline C const *signed_integer_to_string(C *buf, ss_size_t cchBuf, I i)
{
#ifndef __STLSOFT_CF_NEGATIVE_MODULUS_POSITIVE_GIVES_NEGATIVE_RESULT
// If the compiler does not evaluate -9 % 10 to equal -9, then we need to work
// with it as if an unsigned, and prepend the -ve
    C const *psz = unsigned_integer_to_string(buf, cchBuf, i);

    if(i < 0)
    {
        *const_cast<C*>(--psz) = C('-');
    }

    return psz;
#else
// Compiler evaluates -9 % 10 to equal -9, so use the full -ve algorithm. This
// is chosen because it is more efficient on most compilers than calling the
// unsigned peer and converting.

#if defined(__STLSOFT_COMPILER_IS_MSVC)
    typedef I           rem_t;
#else
    typedef ss_sint_t   rem_t;
#endif /* __STLSOFT_COMPILER_IS_MSVC */

    C   *psz    =   buf + cchBuf - 1;  // Set pointer to last character.

    *psz = 0;   // Set the terminating null character.

    if(i < 0)
    {
        do
        {
            rem_t   lsd = static_cast<rem_t>(i % 10);   // Determine the least significant digit.

            i /= 10;                                    // Deal with next most significant.

            --psz;                                      // Move back.

            *psz = get_digit_character<C>()[lsd];

        } while(i != 0);

        *(--psz) = C('-');              // Prepend the minus sign.
    }
    else
    {
        do
        {
            rem_t   lsd = static_cast<rem_t>(i % 10);   // Determine the least significant digit.

            i /= 10;                                    // Deal with next most significant.

            --psz;                                      // Move back.

            *psz = get_digit_character<C>()[lsd];

        } while(i != 0);
    }

    stlsoft_assert(!(psz < buf));

    return psz;
#endif /* !__STLSOFT_CF_NEGATIVE_MODULUS_POSITIVE_GIVES_NEGATIVE_RESULT */
}

/// Highly efficient conversion of signed integers to string
///
/// \note The function does not check that cchBuf is sufficient for conversion
/// of the particular integer type, except for a debug version
/// <code>assert()</code>. It is the programmer's responsibility to ensure that 
/// the supplied buffer is large enough for the conversion. This can be done
/// with the use of the printf_traits templates.
///
/// \param buf A buffer of sufficient space within which to convert the integer
/// \param cchBuf The number of characters available in \c buf
/// \param i The integer to convert
/// \param cchRes Receives the number of characters written
template<   ss_typename_param_k C
        ,   ss_typename_param_k I
        >
inline C const *signed_integer_to_string(C *buf, ss_size_t cchBuf, I i, size_t &cchRes)
{
    C const *psz    =   signed_integer_to_string<C, I>(buf, cchBuf, i);

    cchRes = cchBuf - (psz - (buf - 1));

    return psz;
}

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint8_t i)
{
    return signed_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint8_t i)
{
    return unsigned_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint16_t i)
{
    return signed_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint16_t i)
{
    return unsigned_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint32_t i)
{
    return signed_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint32_t i)
{
    return unsigned_integer_to_string(buf, cchBuf, i);
}

#ifdef __STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT
template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint64_t const &i)
{
#ifdef __STLSOFT_INTEGER_TO_STRING_OPTIMISE_64BIT
    if(i < 0x80000000)
    {
        return signed_integer_to_string(buf, cchBuf, static_cast<ss_sint32_t>(i));
    }
#endif // __STLSOFT_INTEGER_TO_STRING_OPTIMISE_64BIT

    return signed_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint64_t const &i)
{
#ifdef __STLSOFT_INTEGER_TO_STRING_OPTIMISE_64BIT
    if(i < 0x80000000)
    {
        return unsigned_integer_to_string(buf, cchBuf, static_cast<ss_uint32_t>(i));
    }
#endif // __STLSOFT_INTEGER_TO_STRING_OPTIMISE_64BIT

    return unsigned_integer_to_string(buf, cchBuf, i);
}
#endif /* __STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT */

#if 0
template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_int_t i)
{
    return signed_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint_t i)
{
    return unsigned_integer_to_string(buf, cchBuf, i);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_bool_t i);

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_char_a_t i);

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_char_w_t i);
#endif /* 0 */



template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint8_t i, ss_size_t &cchRes)
{
    return signed_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint8_t i, ss_size_t &cchRes)
{
    return unsigned_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint16_t i, ss_size_t &cchRes)
{
    return signed_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint16_t i, ss_size_t &cchRes)
{
    return unsigned_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint32_t i, ss_size_t &cchRes)
{
    return signed_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint32_t i, ss_size_t &cchRes)
{
    return unsigned_integer_to_string(buf, cchBuf, i, cchRes);
}

#ifdef __STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT
template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_sint64_t const &i, ss_size_t &cchRes)
{
    return signed_integer_to_string(buf, cchBuf, i, cchRes);
}

template <ss_typename_param_k C>
inline C const *integer_to_string(C *buf, ss_size_t cchBuf, ss_uint64_t const &i, ss_size_t &cchRes)
{
    return unsigned_integer_to_string(buf, cchBuf, i, cchRes);
}
#endif /* __STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT */

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_INTEGER_TO_STRING */

/* ////////////////////////////////////////////////////////////////////////// */
