/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_pod_veneer.h
 *
 * Purpose:     Contains the auto_destructor and auto_array_destructor template
 *              classes.
 *
 * Created:     19th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_POD_VENEER
#define _STLSOFT_INCL_H_STLSOFT_POD_VENEER

#define _STLSOFT_VER_H_STLSOFT_POD_VENEER_MAJOR       1
#define _STLSOFT_VER_H_STLSOFT_POD_VENEER_MINOR       5
#define _STLSOFT_VER_H_STLSOFT_POD_VENEER_REVISION    1
#define _STLSOFT_VER_H_STLSOFT_POD_VENEER_EDIT        21

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"                // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_CONSTRAINTS
 #include "stlsoft_constraints.h"    // constraints
#endif /* !_STLSOFT_INCL_H_STLSOFT_CONSTRAINTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class pod_veneer

/// Bolts construction and/or destruction to Plain Old Data types
///
/// The parameterisation of this <a href = "http://synesis.com.au/resources/articles/cpp/veneers.pdf">veneer</a>
/// can be used to apply RRID or RAII semantics to a plain old data type. For
/// example,
///
/// &nbsp;&nbsp;<code>struct Simple</code><br>
/// &nbsp;&nbsp;<code>{</code><br>
/// &nbsp;&nbsp;<code>public:</code><br>
/// &nbsp;&nbsp;<code>&nbsp;&nbsp;char&nbsp;*buffer;</code><br>
/// &nbsp;&nbsp;<code>};</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>void Init_Simple(Simple *simple);</code><br>
/// &nbsp;&nbsp;<code>void Uninit_Simple(Simple *simple);</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>struct Simple_Init { void operator ()(Simple *simple) { Init_Simple(simple); } };</code><br>
/// &nbsp;&nbsp;<code>struct Simple_Uninit { void operator ()(Simple *simple) { Uninit_Simple(simple); } };</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>void fn()</code><br>
/// &nbsp;&nbsp;<code>{</code><br>
/// &nbsp;&nbsp;<code>&nbsp;&nbsp;typedef&nbsp;&nbsp;&nbsp;pod_veneer<Simple, Simple_Init, Simple_Uninit>&nbsp;&nbsp;Simple_raii_t;</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>&nbsp;&nbsp;Simple_raii_t&nbsp;simple;&nbsp;// Init_Simple() called here</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>&nbsp;&nbsp;. . .</code><br>
/// &nbsp;&nbsp;<code></code><br>
/// &nbsp;&nbsp;<code>} // Uninit_Simple() called here</code><br>
///
/// \param T The POD type
/// \param CF The function object type applied during construction
/// \param DF The function object type applied during destruction

template<   ss_typename_param_k T
        ,   ss_typename_param_k CF
        ,   ss_typename_param_k DF
        >
class pod_veneer
    : public T
#ifdef __STLSOFT_CF_VENEER_SUPPORT
    , protected is_veneer<T, pod_veneer<T, CF, DF> >
#endif /* __STLSOFT_CF_VENEER_SUPPORT */
{
public:
    /// The value type
    typedef T                       value_type;
    /// The Plain Old Data type
    typedef T                       pod_type;
    /// The constructor function object type
    typedef CF                      constructor_function_type;
    /// The destructor function object type
    typedef DF                      destructor_function_type;
    /// The type of the current parameterisation
    typedef pod_veneer<T, CF, DF>   class_type;

public:
    /// The constructor, within which CF is applied to \c this
    pod_veneer()
    {
        // It is entirely appropriate to constrain instantiations of this class
        // to default construction only, since the whole raison d'etre of this
        // template is to bring RAII semantics to POD types.

        // Construct the pod
        constructor_function_type()(static_cast<pod_type *>(this));
    }

    /// The destructor, within which DF is applied to \c this
    ~pod_veneer() stlsoft_throw_0()
    {
        // There are four important points to note about the destructor
        //
        // (i) It does not throw an exception. Therefore, the destroying type
        // must not throw an exception in its function call operator
        // (ii) It uses the type_is_non_class_or_trivial_class constraint to
        // ensure that the pod_type is either POD or a trivial class
        // (iii) It uses the static assert to ensure that the generated class is
        // the same size as the pod_type, so that arrays of the generated class
        // can be used where arrays of the pod_type could be used. The runtime
        // assert is included for those compilers that do not implement
        // compile-time asserts.
        // (iv) It is non-virtual, since it should not introduce a vtable (which
        // would change the size of the generated class)
        stlsoft_constraint_type_is_non_class_or_trivial_class(pod_type);    // Must be POD
        stlsoft_static_assert(sizeof(class_type) == sizeof(pod_type));      // Must use EBO
#if defined(__STLSOFT_COMPILER_IS_WATCOM)
        stlsoft_assert(sizeof(class_type) == sizeof(pod_type));
#else
        stlsoft_message_assert("pod_veneer used for inappropriate type", sizeof(class_type) == sizeof(pod_type));
#endif /* !__STLSOFT_COMPILER_IS_WATCOM */

        // Destroy the pod
        destructor_function_type()(static_cast<pod_type *>(this));
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_POD_VENEER */

/* ////////////////////////////////////////////////////////////////////////// */
