/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_simple_algorithms.h
 *
 * Purpose:     Contains various simple self-contained algorithms.
 *
 * Created:     17th January 2002
 * Updated:     30th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_SIMPLE_ALGORITHMS
#define _STLSOFT_INCL_H_STLSOFT_SIMPLE_ALGORITHMS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_ALGORITHMS_MAJOR        1
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_ALGORITHMS_MINOR        7
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_ALGORITHMS_REVISION     1
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_ALGORITHMS_EDIT         27
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#include <algorithm>    // std::advance

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Algorithms
 */

// function for_each_if

/// Applies the function to all items in the series for which the predicate is
/// true.   
///
/// \param I The iterator type
/// \param F The function type
/// \param P The predicate type
template<   ss_typename_param_k I
        ,   ss_typename_param_k F
        ,   ss_typename_param_k P
        >
inline F for_each_if(I first, I last, F function, P pred)
{
    for(; first != last; ++first)
    {
        if(pred(*first))
        {
            function(*first);
        }
    }

    return function;
}

// function for_each_count_success

/// Counts the number of items in the sequence which the predicate is true.
///
/// \param I The iterator type
/// \param P The predicate type
template<   ss_typename_param_k I
        ,   ss_typename_param_k P
        >
inline ss_size_t for_each_count_success(I first, I to, P pred)
{
    ss_size_t r;

    for (r = 0; first != to; ++first)
    {
        if(pred(*first))
        {
            ++r;
        }
    }

    return r;
}


// function for_each_set_value

/// Sets the value of all items in the sequence.
///
/// \param I The iterator type
/// \param V The value type
template<   ss_typename_param_k I
        ,   ss_typename_param_k V
        >
inline V const &for_each_set_value(I begin, I end, V const &v)
{
    for(; begin != end; ++begin)
    {
        *begin = v;
    }

    return v;
}


// function for_each_set_value_if

/// Sets the value of all items in the sequence.
///
/// \param I The iterator type
/// \param V The value type
/// \param P The predicate type
template<   ss_typename_param_k I
        ,   ss_typename_param_k V
        ,   ss_typename_param_k P
        >
inline V const &for_each_set_value_if(I begin, I end, V const &v, P pred)
{
    for(; begin != end; ++begin)
    {
        if(pred(*begin))
        {
            *begin = v;
        }
    }

    return v;
}



// function remove_duplicates_from_unordered_sequence

/// This algorithm removes duplicate entries from unordered sequences. It
/// necessarily runs in O(n) time, since it must do a bubble-like double
/// pass on the sequence (in order to work with unordered sequences).
///
/// \param C The container type
/// \param P The predicate type
template<   ss_typename_param_k C
        ,   ss_typename_param_k P
        >
inline void remove_duplicates_from_unordered_sequence(C &container, P pred)
{
    typedef ss_typename_type_k C::iterator  iterator_type;

#if 0
    for(iterator_type begin = container.begin(); begin != container.end(); ++begin)
    {
        iterator_type it = begin;

        for(++it; it != container.end(); ++it)
        {
            if(pred(*begin, *it))
            {
                //
                container.erase(it);
                it = begin;
            }
        }
    }
#endif /* 0 */

    ss_size_t       last;
    ss_size_t       index;
    iterator_type   begin;

    for(index = 0, last = 0, begin = container.begin(); begin != container.end(); )
    {
        iterator_type   it  =   begin;
        iterator_type   end =   container.end();

        if(++it == end)
        {
            ++begin;
        }
        else
        {
            for(;;)
            {
                if(pred(*begin, *it))
                {
                    // Erase the element. We now assume that all iterators
                    // are invalidated
                    container.erase(it);
                    // Remember the last erasure point
                    last = index;
                    // Set begin back to the start of the sequence
                    begin = container.begin();
                    // Move begin to the point where it was at the last erasure
                    stlsoft_ns_qual_std(advance)(begin, last);
                    break;
                }
                else
                {
                    if(++it == end)
                    {
                        ++begin;
                        ++index;
                        break;
                    }
                }
            }
        }
    }

    // Borland seems to think that 'last' is not used, when it clearly is.
    ((void)last);
}



// function for_all
/// Carries out for_each on the range of items in a container
///
/// \param C The container type
/// \param F The function type
template<   ss_typename_param_k C
        ,   ss_typename_param_k F
        >
inline F const &for_all(C &c, F f)
{
    return stlsoft_ns_qual_std(for_each)(c.begin(), c.end(), f);
}



// function for_all_r
/// Carries out for_each on the reverse range of items in a container
///
/// \param C The container type
/// \param F The function type
template<   ss_typename_param_k C
        ,   ss_typename_param_k F
        >
inline F const &for_all_r(C &c, F f)
{
    return stlsoft_ns_qual_std(for_each)(c.rbegin(), c.rend(), f);
}



// function copy_all
/// Carries out for_each on all the items in a container
///
/// \param C The container type
/// \param I The output iterator type
template<   ss_typename_param_k C
        ,   ss_typename_param_k I
        >
inline I const &copy_all(C &c, I i)
{
    return stlsoft_ns_qual_std(copy)(c.begin(), c.end(), i);
}



// function for_each_preinc
/// Carries out for_each on the range, using pre-increment on the iterator
///
/// \param I The iterator type
/// \param F The function type
template<   ss_typename_param_k I
        ,   ss_typename_param_k F
        >
inline F for_each_preinc(I first, I last, F f)
{
    for(; first != last; ++first)
    {
        f(*first);
    }

    return f;
}


// function for_each_postinc
/// Carries out for_each on the range, using post-increment on the iterator
///
/// \param I The iterator type
/// \param F The function type
template<   ss_typename_param_k I
        ,   ss_typename_param_k F
        >
inline F for_each_postinc(I first, I last, F f)
{
    for(; first != last; first++)
    {
        f(*first);
    }

    return f;
}


/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_SIMPLE_ALGORITHMS */

/* ////////////////////////////////////////////////////////////////////////// */
