/* /////////////////////////////////////////////////////////////////////////////
 * File:        unixstl_findfile_sequence.h
 *
 * Purpose:     findfile_sequence class.
 *
 * Created:     15th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/unixstl
 *                          http://www.unixstl.org/
 *
 *              email:      submissions@unixstl.org  for submissions
 *                          admin@unixstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _INCL_UNIXSTL_H_UNIXSTL_FINDFILE_SEQUENCE
#define _INCL_UNIXSTL_H_UNIXSTL_FINDFILE_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _UNIXSTL_VER_H_UNIXSTL_FINDFILE_SEQUENCE_MAJOR      1
#define _UNIXSTL_VER_H_UNIXSTL_FINDFILE_SEQUENCE_MINOR      5
#define _UNIXSTL_VER_H_UNIXSTL_FINDFILE_SEQUENCE_REVISION   6
#define _UNIXSTL_VER_H_UNIXSTL_FINDFILE_SEQUENCE_EDIT       38
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _INCL_UNIXSTL_H_UNIXSTL
 #include "unixstl.h"               // Include the UNIXSTL root header
#endif /* !_INCL_UNIXSTL_H_UNIXSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */
#include <glob.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The UNIXSTL components are contained within the unixstl namespace. This is
 * actually an alias for stlsoft::unixstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _UNIXSTL_NO_NAMESPACE   unixstl definition
 * ---------------------    ---------------------   -----------------
 *  not defined              not defined             = stlsoft::unixstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             unixstl
 *  defined                  defined                 not defined
 *
 */

/* No STLSoft namespaces means no UNIXSTL namespaces */
#ifdef _STLSOFT_NO_NAMESPACES
 #define _UNIXSTL_NO_NAMESPACES
#endif /* _STLSOFT_NO_NAMESPACES */

/* No UNIXSTL namespaces means no unixstl namespace */
#ifdef _UNIXSTL_NO_NAMESPACES
 #define _UNIXSTL_NO_NAMESPACE
#endif /* _UNIXSTL_NO_NAMESPACES */

#ifndef _UNIXSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::unixstl */
namespace unixstl
{
 #else
/* Define stlsoft::unixstl_project */

namespace stlsoft
{

namespace unixstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_UNIXSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \defgroup unixstl_filesystem_library File-System Library (UNIXSTL)
/// \ingroup UNIXSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating UNIX file-system objects
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// \brief STL-like readonly sequence based on the results of file-system wildcard matches
///
/// This class presents and STL-like readonly sequence interface to allow the 
/// iteration over the results of file-system wildcard matches.

class findfile_sequence
{
private:
    typedef findfile_sequence   class_type;
public:
    /// The value type
    typedef us_char_a_t const   *value_type;
    /// The type of the const (non-mutating) iterator
    typedef value_type          *const_iterator;
private:
//    typedef value_type          &reference;
    typedef value_type const    &const_reference;
//    typedef value_type          *pointer;
    typedef value_type const    *const_pointer;
public:
    /// The size type
    typedef us_size_t           size_type;
    /// The difference type
    typedef us_ptrdiff_t        difference_type;

    /// The type of the const (non-mutating) reverse iterator
    typedef stlsoft_ns_qual(reverse_iterator_base)  <   const_iterator
                                                    ,   value_type
                                                    ,   const_reference
                                                    ,   const_pointer
                                                    ,   difference_type
                                                    >   const_reverse_iterator;

public:
    enum
    {
            includeDots =   0x0008
        ,   onlyDir     =   0x0010
        ,   noSort      =   0x0100
        ,   markDirs    =   0x0200
        ,
    };

// Construction
public:
    /// \brief Constructs a sequence according to the given criteria
    ///
    /// The constructor initialises a findfile_sequence instance on the given
    /// pattern with the given flags.
    ///
    /// \param pattern The pattern against which to match the file-system contents
    /// \param flags Flags to alter the behaviour of the search
    findfile_sequence(us_char_a_t const *pattern, us_int_t flags = noSort)
        : m_flags(_validate_flags(flags))
    {
        us_int_t    glob_flags  =   0;

        if(flags & noSort)
        {
            glob_flags |= GLOB_NOSORT;
        }

        if(flags & markDirs)
        {
            glob_flags |= GLOB_MARK;
        }

        if(flags & onlyDir)
        {
            glob_flags |= GLOB_ONLYDIR;
        }

        if(glob(pattern, glob_flags, 0, &m_gl) == 0)
        {
            us_char_a_t **base  =   m_gl.gl_pathv;
            us_int_t    cItems  =   m_gl.gl_pathc;

            if(!(flags & includeDots))
            {
                // Now remove the dots. If located at the start of
                // the gl buffer, then simply increment m_base to
                // be above that. If not then rearrange the base
                // two pointers such that they are there.

                us_bool_t   foundDot1   =   false;
                us_bool_t   foundDot2   =   false;
                us_char_a_t **begin     =   base;
                us_char_a_t **end       =   begin + cItems;

                for(; begin != end; ++begin)
                {
                    us_bool_t   bTwoDots;

                    if(_is_dots(*begin, bTwoDots))
                    {
                        if(begin != base)
                        {
                            // Now need to swap with whatever is at base[0]
                            us_char_a_t *t  =   *begin;

                            *begin  = *base;
                            *base   = t;
                        }

                        ++base;
                        --cItems;

                        (bTwoDots ? foundDot2 : foundDot1) = true;

                        if( foundDot1 &&
                            foundDot2)
                        {
                            break;
                        }
                    }
                }
            }

            // Set m_base and m_cItems to the correct values, with
            // or without dots. m_base is cast here to remove the
            // need for const-casting throughout the rest of the
            // class
            m_base      =   const_cast<us_char_a_t const **>(base);
            m_cItems    =   cItems;
        }
        else
        {
            m_base = 0;
            m_cItems = 0;
        }
    }

    // Releases any acquired resources
    ~findfile_sequence() unixstl_throw_0()
    {
        if(m_base != 0)
        {
            globfree(&m_gl);
        }
    }

// Attributes
public:
    /// Returns the number of elements in the sequence
    us_size_t size() const
    {
        return m_cItems;
    }

    /// \brief Indicates whether the search sequence is empty
    us_bool_t empty() const
    {
        return size() == 0;
    }

    /// \brief Returns the value corresponding to the given index
    ///
    /// \note In debug-mode a runtime assert is applied to enforce that the index is valid. There is <b>no</b> release-time checking on the index validity!
    value_type const operator [](size_type index) const
    {
        unixstl_message_assert("index access out of range in findfile_sequence", index < m_cItems + 1);   // Has to be +1, since legitimate to take address of one-past-the-end

        return m_base[index];
    }

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const
    {
        return m_base;
    }
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const
    {
        return m_base + m_cItems;
    }

    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    const_reverse_iterator  rbegin() const
    {
        return const_reverse_iterator(end());
    }
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    const_reverse_iterator  rend() const
    {
        return const_reverse_iterator(begin());
    }

// Implementation
private:
    static us_int_t _validate_flags(us_int_t flags)
    {
        return flags & (includeDots | onlyDir | noSort | markDirs);
    }

    // Returns true if pch == "" or "/" (or "\\"), false otherwise
    us_bool_t _is_end_of_path_elements(us_char_a_t const *pch, difference_type index)
    {
        return  pch[index] == '\0' ||
                (   pch[index + 1] == '\0' &&
                    (
#if defined(_UNIXSTL_COMPILER_IS_UNKNOWN) && \
    !defined(_UNIXSTL_FINDFILE_SEQUENCE_NO_BACK_SLASH_TERMINATOR)
                        pch[index] == '\\' ||
#endif /* _UNIXSTL_COMPILER_IS_UNKNOWN && !_UNIXSTL_FINDFILE_SEQUENCE_NO_BACK_SLASH_TERMINATOR */
                        pch[index] == '/'));
    }

    us_bool_t _is_dots(us_char_a_t const *s, us_bool_t &bTwoDots)
    {
        return  s != 0 &&
                s[0] == '.' &&
                (   (bTwoDots = false, _is_end_of_path_elements(s, 1)) ||
                    (bTwoDots = true, ( s[1] == '.' &&
                                        _is_end_of_path_elements(s, 2))));
    }

// Members
private:
    us_int_t            m_flags;
    glob_t              m_gl;
    us_char_a_t const   **m_base;
    us_int_t            m_cItems;

// Not to be implemented
private:
    findfile_sequence(class_type const &);
    class_type const &operator =(class_type const &);
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group unixstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _UNIXSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace unixstl
 #else
} // namespace unixstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_UNIXSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_INCL_UNIXSTL_H_UNIXSTL_FINDFILE_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
