/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_string_access.h
 *
 * Purpose:     Contains classes and functions for dealing with Win32 strings.
 *
 * Created:
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_STRING_ACCESS
#define _WINSTL_INCL_H_WINSTL_STRING_ACCESS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_STRING_ACCESS_MAJOR     1
#define _WINSTL_VER_H_WINSTL_STRING_ACCESS_MINOR     5
#define _WINSTL_VER_H_WINSTL_STRING_ACCESS_REVISION  3
#define _WINSTL_VER_H_WINSTL_STRING_ACCESS_EDIT      40
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_string_access String Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types

/// \defgroup winstl_string_access_shims String Access Shims (WinSTL)
/// \ingroup WinSTL concepts_shims_string_access
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/* HWND */
/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the window text of a given window may be
/// accessed as a null-terminated string.
template <ws_typename_param_k C>
class c_str_ptr_null_HWND_proxy
{
public:
    typedef C                                           char_type;
    typedef c_str_ptr_null_HWND_proxy<C>                class_type;
    typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given HWND
    ///
    /// \param h The HWND from which the text will be retrieved
    ws_explicit_k c_str_ptr_null_HWND_proxy(HWND h)
    {
        int length  =   ::GetWindowTextLength(h);

        if(length == 0)
        {
            m_buffer = NULL;
        }
        else
        {
            m_buffer = new char_type[1 + length];

            get_window_text(h, m_buffer, length + 1);
        }
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_null_HWND_proxy(rhs_type rhs)
        : m_buffer(rhs.m_buffer)
    {
        move_lhs_from_rhs(rhs).m_buffer = NULL;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_null_HWND_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the window contents, or
    /// the empty string "" if the window contains no text.
    operator char_type const *() const
    {
        return m_buffer;
    }

// Implementation
private:
    int get_window_text(HWND h, char_type *buffer, int cchBuffer);

// Members
private:
    char_type   *m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline int c_str_ptr_null_HWND_proxy<ws_char_a_t>::get_window_text(HWND h, ws_char_a_t *buffer, int cchBuffer)
{
    return ::GetWindowTextA(h, buffer, cchBuffer);
}

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline int c_str_ptr_null_HWND_proxy<ws_char_w_t>::get_window_text(HWND h, ws_char_w_t *buffer, int cchBuffer)
{
    return ::GetWindowTextW(h, buffer, cchBuffer);
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/// This class provides an intermediary object that may be returned by the
/// c_str_ptr() function, such that the window text of a given window may be
/// accessed as a null-terminated string.
template <ws_typename_param_k C>
class c_str_ptr_HWND_proxy
{
public:
    typedef C                                           char_type;
    typedef c_str_ptr_HWND_proxy<C>                     class_type;
    typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given HWND
    ///
    /// \param h The HWND from which the text will be retrieved
    ws_explicit_k c_str_ptr_HWND_proxy(HWND h)
    {
        int length  =   ::GetWindowTextLength(h);

        m_buffer = new char_type[1 + length];

        get_window_text(h, m_buffer, length + 1);
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_HWND_proxy(rhs_type rhs)
        : m_buffer(rhs.m_buffer)
    {
        move_lhs_from_rhs(rhs).m_buffer = NULL;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_HWND_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the window contents, or
    /// the empty string "" if the window contains no text.
    operator char_type const *() const
    {
        return m_buffer;
    }

// Implementation
private:
    int get_window_text(HWND h, char_type *buffer, int cchBuffer);

// Members
private:
    char_type   *m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline int c_str_ptr_HWND_proxy<ws_char_a_t>::get_window_text(HWND h, ws_char_a_t *buffer, int cchBuffer)
{
    return ::GetWindowTextA(h, buffer, cchBuffer);
}

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
inline int c_str_ptr_HWND_proxy<ws_char_w_t>::get_window_text(HWND h, ws_char_w_t *buffer, int cchBuffer)
{
    return ::GetWindowTextW(h, buffer, cchBuffer);
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */


/* Windows Policy Management LSA_UNICODE_STRING string */
#ifdef _NTSECAPI_
/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the text of a given LSA_UNICODE_STRING
/// string may be accessed as a null-terminated string.
class c_str_ptr_LSA_UNICODE_STRING_proxy
{
public:
    typedef c_str_ptr_LSA_UNICODE_STRING_proxy  class_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given LSA_UNICODE_STRING instance
    ///
    /// \param h The LSA_UNICODE_STRING instance from which the text will be retrieved
    ws_explicit_k c_str_ptr_LSA_UNICODE_STRING_proxy(const LSA_UNICODE_STRING &s)
        : m_buffer(new WCHAR[1 + s.Length])
    {
        wcsncpy(m_buffer, s.Buffer, s.Length);
        m_buffer[s.Length] = L'\0';
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_LSA_UNICODE_STRING_proxy(class_type &rhs)
        : m_buffer(rhs.m_buffer)
    {
        rhs.m_buffer = NULL;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_LSA_UNICODE_STRING_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the string contents, or
    /// the empty string "" if the string has no contents.
    operator LPCWSTR () const
    {
        return m_buffer;
    }

// Members
private:
    LPWSTR  m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};
#endif /* _NTSECAPI_ */

#ifdef _NTSECAPI_
/// This class provides an intermediary object that may be returned by the
/// c_str_ptr_null() function, such that the text of a given LSA_UNICODE_STRING
/// string may be accessed as a null-terminated string.
class c_str_ptr_null_LSA_UNICODE_STRING_proxy
{
public:
    typedef c_str_ptr_null_LSA_UNICODE_STRING_proxy class_type;

// Construction
public:
    /// Constructs an instance of the proxy from the given LSA_UNICODE_STRING instance
    ///
    /// \param h The LSA_UNICODE_STRING instance from which the text will be retrieved
    ws_explicit_k c_str_ptr_null_LSA_UNICODE_STRING_proxy(const LSA_UNICODE_STRING &s)
        : m_buffer((s.Length != 0) ? new WCHAR[1 + s.Length] : NULL)
    {
        if(m_buffer != NULL)
        {
            wcsncpy(m_buffer, s.Buffer, s.Length);
            m_buffer[s.Length] = L'\0';
        }
    }

    /// Move constructor
    /// 
    /// This <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move constructor</a>
    /// is for circumstances when the compiler does not, or cannot, apply the
    /// return value optimisation. It causes the contents of \c rhs to be
    /// transferred into the constructing instance. This is completely safe
    /// because the \c rhs instance will never be accessed in its own right, so
    /// does not need to maintain ownership of its contents.
    c_str_ptr_null_LSA_UNICODE_STRING_proxy(class_type &rhs)
        : m_buffer(rhs.m_buffer)
    {
        rhs.m_buffer = NULL;
    }

    /// Releases any storage aquired by the proxy
    ~c_str_ptr_null_LSA_UNICODE_STRING_proxy()
    {
        delete [] m_buffer;
    }

// Accessors
public:
    /// Returns a null-terminated string representing the string contents, or
    /// NULL if the string has no contents.
    operator LPCWSTR () const
    {
        return m_buffer;
    }

// Members
private:
    LPWSTR  m_buffer;

// Not to be implemented
private:
    void operator =(class_type const &rhs);
};
#endif /* _NTSECAPI_ */

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr_null
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or NULL.
 */

/* HWND */
/// \brief Returns the corresponding C-string pointer of the window \c h, or a null pointer
inline c_str_ptr_null_HWND_proxy<ws_char_a_t> c_str_ptr_null_a(HWND h)
{
    return c_str_ptr_null_HWND_proxy<ws_char_a_t>(h);
}

/// \brief Returns the corresponding C-string pointer of the window \c h, or a null pointer
inline c_str_ptr_null_HWND_proxy<ws_char_w_t> c_str_ptr_null_w(HWND h)
{
    return c_str_ptr_null_HWND_proxy<ws_char_w_t>(h);
}

/// \brief Returns the corresponding C-string pointer of the window \c h, or a null pointer
inline c_str_ptr_null_HWND_proxy<TCHAR> c_str_ptr_null(HWND h)
{
    return c_str_ptr_null_HWND_proxy<TCHAR>(h);
}

/* LSA_UNICODE_STRING */
#ifdef _NTSECAPI_
/// \brief Returns the corresponding C-string pointer of the LSA_UNICODE_STRING \c s, or a null pointer
inline c_str_ptr_null_LSA_UNICODE_STRING_proxy c_str_ptr_null(const LSA_UNICODE_STRING &s)
{
    return c_str_ptr_null_LSA_UNICODE_STRING_proxy(s);
}
#endif /* _NTSECAPI_ */

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_ptr
 *
 * This can be applied to an expression, and the return value is either a
 * pointer to the character string or to an empty string.
 */

/* HWND */
/// \brief Returns the corresponding C-string pointer of the window \c h
inline c_str_ptr_HWND_proxy<ws_char_a_t> c_str_ptr_a(HWND h)
{
    return c_str_ptr_HWND_proxy<ws_char_a_t>(h);
}

/// \brief Returns the corresponding C-string pointer of the window \c h
inline c_str_ptr_HWND_proxy<ws_char_w_t> c_str_ptr_w(HWND h)
{
    return c_str_ptr_HWND_proxy<ws_char_w_t>(h);
}

/// \brief Returns the corresponding C-string pointer of the window \c h
inline c_str_ptr_HWND_proxy<TCHAR> c_str_ptr(HWND h)
{
    return c_str_ptr_HWND_proxy<TCHAR>(h);
}

/* LSA_UNICODE_STRING */
#ifdef _NTSECAPI_
/// \brief Returns the corresponding C-string pointer of the LSA_UNICODE_STRING \c s
inline c_str_ptr_LSA_UNICODE_STRING_proxy c_str_ptr(const LSA_UNICODE_STRING &s)
{
    return c_str_ptr_LSA_UNICODE_STRING_proxy(s);
}
#endif /* _NTSECAPI_ */

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_len
 *
 * This can be applied to an expression, and the return value is the number of
 * characters in the character string in the expression.
 */

/* HWND */
/// \brief Returns the length (in characters) of the string of \c s, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_len(HWND h)
{
    return static_cast<ws_size_t>(::GetWindowTextLength(h));
}

/// \brief Returns the length (in characters) of the string of \c s, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_len_a(HWND h)
{
    return c_str_len(h);
}

/// \brief Returns the length (in characters) of the string of \c s, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_len_w(HWND h)
{
    return c_str_len(h);
}

/* LSA_UNICODE_STRING */
#ifdef _NTSECAPI_
/// \brief Returns the length (in characters) of the LSA_UNICODE_STRING \c s, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_len(const LSA_UNICODE_STRING &s)
{
    return s.Length;
}
#endif /* _NTSECAPI_ */

/* /////////////////////////////////////////////////////////////////////////////
 * c_str_size
 *
 * This can be applied to an expression, and the return value is the number of
 * bytes required to store the character string in the expression, NOT including
 * the null-terminating character.
 */

/* HWND */
/// \brief Returns the size (in bytes) of the string of \c h, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_size_a(HWND h)
{
    return c_str_len(h) * sizeof(ws_char_a_t);
}

/// \brief Returns the size (in bytes) of the string of \c h, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_size_w(HWND h)
{
    return c_str_len(h) * sizeof(ws_char_w_t);
}

/// \brief Returns the size (in bytes) of the string of \c h, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_size(HWND h)
{
    return c_str_len(h) * sizeof(TCHAR);
}

/* LSA_UNICODE_STRING */
#ifdef _NTSECAPI_
/// \brief Returns the size (in bytes) of the LSA_UNICODE_STRING \c s, <b><i>not</i></b> including the null-terminating character
inline ws_size_t c_str_size(const LSA_UNICODE_STRING &s)
{
    return c_str_len(s) * sizeof(WCHAR);
}
#endif /* _NTSECAPI_ */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_string_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_SYSTEM_INFO */

/* ////////////////////////////////////////////////////////////////////////// */
